# Copyright 2013 Big Switch Networks Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.fw import firewallrule
from neutronclient.tests.unit import test_cli20


class CLITestV20FirewallRuleJSON(test_cli20.CLITestV20Base):

    def _test_create_firewall_rule_with_mandatory_params(self, enabled):
        # firewall-rule-create with mandatory (none) params only.
        resource = 'firewall_rule'
        cmd = firewallrule.CreateFirewallRule(test_cli20.MyApp(sys.stdout),
                                              None)
        tenant_id = 'my-tenant'
        name = ''
        my_id = 'myid'
        protocol = 'tcp'
        action = 'allow'
        ip_version = 4
        args = ['--tenant-id', tenant_id,
                '--admin-state-up',
                '--protocol', protocol,
                '--action', action,
                '--enabled', enabled]
        position_names = []
        position_values = []
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   protocol=protocol, action=action,
                                   enabled=enabled, tenant_id=tenant_id,
                                   ip_version=ip_version)

    def test_create_enabled_firewall_rule_with_mandatory_params_lcase(self):
        self._test_create_firewall_rule_with_mandatory_params(enabled='true')

    def test_create_disabled_firewall_rule_with_mandatory_params_lcase(self):
        self._test_create_firewall_rule_with_mandatory_params(enabled='false')

    def test_create_enabled_firewall_rule_with_mandatory_params(self):
        self._test_create_firewall_rule_with_mandatory_params(enabled='True')

    def test_create_disabled_firewall_rule_with_mandatory_params(self):
        self._test_create_firewall_rule_with_mandatory_params(enabled='False')

    def _setup_create_firewall_rule_with_all_params(
            self, protocol='tcp', protocol_cli=None,
            action='allow', action_cli=None, ip_version='4'):
        # firewall-rule-create with all params set.
        resource = 'firewall_rule'
        cmd = firewallrule.CreateFirewallRule(test_cli20.MyApp(sys.stdout),
                                              None)
        name = 'my-name'
        description = 'my-desc'
        source_ip = '192.168.1.0/24'
        destination_ip = '192.168.2.0/24'
        source_port = '0:65535'
        destination_port = '0:65535'
        tenant_id = 'my-tenant'
        my_id = 'myid'
        enabled = 'True'
        args = ['--description', description,
                '--shared',
                '--protocol', protocol_cli or protocol,
                '--ip-version', ip_version,
                '--source-ip-address', source_ip,
                '--destination-ip-address', destination_ip,
                '--source-port', source_port,
                '--destination-port', destination_port,
                '--action', action_cli or action,
                '--enabled', enabled,
                '--admin-state-up',
                '--tenant-id', tenant_id]
        position_names = []
        position_values = []
        if protocol == 'any':
            protocol = None
        if ip_version == '4' or ip_version == '6':
            self._test_create_resource(resource, cmd, name, my_id, args,
                                       position_names, position_values,
                                       description=description, shared=True,
                                       protocol=protocol,
                                       ip_version=int(ip_version),
                                       source_ip_address=source_ip,
                                       destination_ip_address=destination_ip,
                                       source_port=source_port,
                                       destination_port=destination_port,
                                       action=action, enabled='True',
                                       tenant_id=tenant_id)
        else:
            self.assertRaises(SystemExit, self._test_create_resource,
                              resource, cmd, name, my_id, args,
                              position_names, position_values,
                              ip_version=int(ip_version),
                              source_ip_address=source_ip,
                              destination_ip_address=destination_ip,
                              source_port=source_port,
                              destination_port=destination_port,
                              action=action, enabled='True',
                              tenant_id=tenant_id)

    def test_create_firewall_rule_with_all_params(self):
        self._setup_create_firewall_rule_with_all_params()

    def test_create_firewall_rule_with_proto_any(self):
        self._setup_create_firewall_rule_with_all_params(protocol='any')

    def test_create_firewall_rule_with_IP_version_6(self):
        self._setup_create_firewall_rule_with_all_params(ip_version='6')

    def test_create_firewall_rule_with_invalid_IP_version(self):
        self._setup_create_firewall_rule_with_all_params(ip_version='5')

    def test_create_firewall_rule_with_proto_action_upper_capitalized(self):
        for protocol in ('TCP', 'Tcp', 'ANY', 'AnY'):
            self._setup_create_firewall_rule_with_all_params(
                protocol=protocol.lower(),
                protocol_cli=protocol)
        for action in ('Allow', 'DENY', 'reject'):
            self._setup_create_firewall_rule_with_all_params(
                action=action.lower(),
                action_cli=action)

    def test_list_firewall_rules(self):
        # firewall-rule-list.
        resources = "firewall_rules"
        cmd = firewallrule.ListFirewallRule(test_cli20.MyApp(sys.stdout),
                                            None)
        self._test_list_resources(resources, cmd, True)

    def test_list_firewall_rules_pagination(self):
        # firewall-rule-list.
        resources = "firewall_rules"
        cmd = firewallrule.ListFirewallRule(test_cli20.MyApp(sys.stdout),
                                            None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_firewall_rules_sort(self):
        # firewall-rule-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "firewall_rules"
        cmd = firewallrule.ListFirewallRule(test_cli20.MyApp(sys.stdout),
                                            None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_firewall_rules_limit(self):
        # firewall-rule-list -P."""
        resources = "firewall_rules"
        cmd = firewallrule.ListFirewallRule(test_cli20.MyApp(sys.stdout),
                                            None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_firewall_rule_id(self):
        # firewall-rule-show test_id.
        resource = 'firewall_rule'
        cmd = firewallrule.ShowFirewallRule(test_cli20.MyApp(sys.stdout),
                                            None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_firewall_rule_id_name(self):
        # firewall-rule-show.
        resource = 'firewall_rule'
        cmd = firewallrule.ShowFirewallRule(test_cli20.MyApp(sys.stdout),
                                            None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_firewall_rule(self):
        # firewall-rule-update myid --name newname.
        resource = 'firewall_rule'
        cmd = firewallrule.UpdateFirewallRule(test_cli20.MyApp(sys.stdout),
                                              None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname'],
                                   {'name': 'newname', })

        # firewall-rule-update myid --protocol any.
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--protocol', 'any'],
                                   {'protocol': None, })

        # firewall-rule-update myid --description any
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--description', 'any'],
                                   {'description': 'any', })

        # firewall-rule-update myid --source_ip_address 192.192.192.192
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--source_ip_address',
                                    '192.192.192.192'],
                                   {'source_ip_address': '192.192.192.192', })

        # firewall-rule-update myid --source_port 32767
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--source_port', '32767'],
                                   {'source_port': '32767', })

        # firewall-rule-update myid --destination_ip_address 0.1.0.1
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--destination_ip_address',
                                    '0.1.0.1'],
                                   {'destination_ip_address': '0.1.0.1', })

        # firewall-rule-update myid --destination_port 65432
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--destination_port',
                                    '65432'],
                                   {'destination_port': '65432', })

        # firewall-rule-update myid --enabled  False
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--enabled', 'False'],
                                   {'enabled': 'False', })

        # firewall-rule-update myid --action reject
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--action', 'reject'],
                                   {'action': 'reject', })

        # firewall-rule-update myid --shared false
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--shared', 'false'],
                                   {'shared': 'false', })

    def test_delete_firewall_rule(self):
        # firewall-rule-delete my-id.
        resource = 'firewall_rule'
        cmd = firewallrule.DeleteFirewallRule(test_cli20.MyApp(sys.stdout),
                                              None)
        my_id = 'myid1'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



# Copyright 2013 Big Switch Networks Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.neutron.v2_0.fw import firewallpolicy
from neutronclient import shell
from neutronclient.tests.unit import test_cli20


class CLITestV20FirewallPolicyJSON(test_cli20.CLITestV20Base):
    def setUp(self):
        super(CLITestV20FirewallPolicyJSON, self).setUp()

    def test_create_firewall_policy_with_mandatory_params(self):
        # firewall-policy-create with mandatory (none) params only.
        resource = 'firewall_policy'
        cmd = firewallpolicy.CreateFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                  None)
        tenant_id = 'my-tenant'
        name = 'my-name'
        my_id = 'myid'
        args = ['--tenant-id', tenant_id,
                '--admin-state_up',
                name, ]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   admin_state_up=True, tenant_id=tenant_id)

    def test_create_firewall_policy_with_all_params(self):
        # firewall-policy-create with rule param of misc format.
        resource = 'firewall_policy'
        cmd = firewallpolicy.CreateFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                  None)
        name = 'my-name'
        description = 'my-desc'
        firewall_rules_res = ['rule_id1', 'rule_id2']
        tenant_id = 'my-tenant'
        my_id = 'myid'
        position_names = ['name', ]
        position_values = [name, ]

        # check for both str and unicode format firewall_rules_arg
        for firewall_rules_arg in ['rule_id1 rule_id2', u'rule_id1 rule_id2']:
            args = ['--description', description,
                    '--shared',
                    '--firewall-rules', firewall_rules_arg,
                    '--audited',
                    '--tenant-id', tenant_id,
                    '--admin-state_up',
                    name]
            self._test_create_resource(resource, cmd, name, my_id, args,
                                       position_names, position_values,
                                       description=description, shared=True,
                                       firewall_rules=firewall_rules_res,
                                       audited=True, admin_state_up=True,
                                       tenant_id=tenant_id)

    def test_list_firewall_policies(self):
        # firewall-policy-list.
        resources = "firewall_policies"
        cmd = firewallpolicy.ListFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                None)
        self._test_list_resources(resources, cmd, True)

    def test_list_firewall_policies_pagination(self):
        # firewall-policy-list."""
        resources = "firewall_policies"
        cmd = firewallpolicy.ListFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_firewall_policies_sort(self):
        # sorted list: firewall-policy-list --sort-key name --sort-key id
        # --sort-key asc --sort-key desc
        resources = "firewall_policies"
        cmd = firewallpolicy.ListFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_firewall_policies_limit(self):
        # size (1000) limited list: firewall-policy-list -P.
        resources = "firewall_policies"
        cmd = firewallpolicy.ListFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_firewall_policy_id(self):
        # firewall-policy-show test_id.
        resource = 'firewall_policy'
        cmd = firewallpolicy.ShowFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_firewall_policy_id_name(self):
        # firewall-policy-show.
        resource = 'firewall_policy'
        cmd = firewallpolicy.ShowFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_firewall_policy(self):
        # firewall-policy-update myid --name newname.
        resource = 'firewall_policy'
        cmd = firewallpolicy.UpdateFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                  None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname'],
                                   {'name': 'newname', })

    def test_update_firewall_policy_with_rules(self):
        # firewall-policy-update myid --firewall-rules "rule1 rule2".
        resource = 'firewall_policy'
        cmd = firewallpolicy.UpdateFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                  None)
        firewall_rules_arg = u'rule_id3 rule_id4'
        firewall_rules_res = ['rule_id3', 'rule_id4']
        self._test_update_resource(
            resource, cmd, 'myid',
            ['myid', '--firewall-rules', firewall_rules_arg],
            {'firewall_rules': firewall_rules_res, })

    def test_delete_firewall_policy(self):
        # firewall-policy-delete my-id.
        resource = 'firewall_policy'
        cmd = firewallpolicy.DeleteFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                  None)
        my_id = 'myid1'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_insert_firewall_rule(self):
        # firewall-policy-insert-rule myid newruleid --insert-before ruleAid
        # --insert-after ruleBid
        resource = 'firewall_policy'
        cmd = firewallpolicy.FirewallPolicyInsertRule(
            test_cli20.MyApp(sys.stdout),
            None)
        myid = 'myid'
        args = ['myid', 'newrule',
                '--insert-before', 'rule2',
                '--insert-after', 'rule1']
        extrafields = {'firewall_rule_id': 'newrule',
                       'insert_before': 'rule2',
                       'insert_after': 'rule1'}

        body = extrafields
        path = getattr(self.client, resource + "_insert_path")
        cmd_parser = cmd.get_parser(resource + "_insert_rule")
        resp = (test_cli20.MyResp(204), None)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            shell.run_command(cmd, cmd_parser, args)
        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), 4)
        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path % myid),
                self.client),
            'PUT', body=test_cli20.MyComparator(body, self.client),
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

    def test_remove_firewall_rule(self):
        # firewall-policy-remove-rule myid ruleid
        resource = 'firewall_policy'
        cmd = firewallpolicy.FirewallPolicyRemoveRule(
            test_cli20.MyApp(sys.stdout),
            None)
        myid = 'myid'
        args = ['myid', 'removerule']
        extrafields = {'firewall_rule_id': 'removerule', }

        body = extrafields
        path = getattr(self.client, resource + "_remove_path")
        cmd_parser = cmd.get_parser(resource + "_remove_rule")
        resp = (test_cli20.MyResp(204), None)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            shell.run_command(cmd, cmd_parser, args)
        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), 2)
        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(
                test_cli20.end_url(path % myid),
                self.client),
            'PUT', body=test_cli20.MyComparator(body, self.client),
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

    def test_update_firewall_policy_name_shared_audited(self):
        # firewall-policy-update myid --name newname2 --shared --audited
        resource = 'firewall_policy'
        cmd = firewallpolicy.UpdateFirewallPolicy(test_cli20.MyApp(sys.stdout),
                                                  None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname2',
                                    '--shared', 'True', '--audited', 'True'],
                                   {'name': 'newname2',
                                    'shared': 'True', 'audited': 'True'})



# Copyright (c) 2017 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from neutronclient.osc.v2.sfc import sfc_port_pair_group
from neutronclient.tests.unit.osc.v2.sfc import fakes


def _get_id(client, id_or_name, resource):
    return id_or_name


class TestCreateSfcPortPairGroup(fakes.TestNeutronClientOSCV2):

    _port_pair_group = fakes.FakeSfcPortPairGroup.create_port_pair_group()

    columns = ('Description',
               'ID',
               'Loadbalance ID',
               'Name',
               'Port Pair',
               'Port Pair Group Parameters',
               'Project',
               'Tap Enabled')

    def get_data(self, ppg):
        return (
            ppg['description'],
            ppg['id'],
            ppg['group_id'],
            ppg['name'],
            ppg['port_pairs'],
            ppg['port_pair_group_parameters'],
            ppg['project_id'],
            ppg['tap_enabled']
        )

    def setUp(self):
        super(TestCreateSfcPortPairGroup, self).setUp()
        mock.patch(
            'neutronclient.osc.v2.sfc.sfc_port_pair_group._get_id',
            new=_get_id).start()
        self.neutronclient.create_sfc_port_pair_group = mock.Mock(
            return_value={'port_pair_group': self._port_pair_group})
        self.data = self.get_data(self._port_pair_group)
        # Get the command object to test
        self.cmd = sfc_port_pair_group.CreateSfcPortPairGroup(self.app,
                                                              self.namespace)

    def test_create_port_pair_group_default_options(self):
        arglist = [
            "--port-pair", self._port_pair_group['port_pairs'],
            self._port_pair_group['name'],
        ]
        verifylist = [
            ('port_pairs', [self._port_pair_group['port_pairs']]),
            ('name', self._port_pair_group['name']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))
        self.neutronclient.create_sfc_port_pair_group.assert_called_once_with({
            'port_pair_group': {
                'name': self._port_pair_group['name'],
                'port_pairs': [self._port_pair_group['port_pairs']]
            }
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_port_pair_group(self):
        arglist = [
            "--description", self._port_pair_group['description'],
            "--port-pair", self._port_pair_group['port_pairs'],
            self._port_pair_group['name'],
        ]
        verifylist = [
            ('port_pairs', [self._port_pair_group['port_pairs']]),
            ('name', self._port_pair_group['name']),
            ('description', self._port_pair_group['description']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.neutronclient.create_sfc_port_pair_group.assert_called_once_with({
            'port_pair_group': {
                'name': self._port_pair_group['name'],
                'port_pairs': [self._port_pair_group['port_pairs']],
                'description': self._port_pair_group['description'],
            }
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create_tap_enabled_port_pair_group(self):
        arglist = [
            "--description", self._port_pair_group['description'],
            "--port-pair", self._port_pair_group['port_pairs'],
            self._port_pair_group['name'],
            "--enable-tap"
        ]
        verifylist = [
            ('port_pairs', [self._port_pair_group['port_pairs']]),
            ('name', self._port_pair_group['name']),
            ('description', self._port_pair_group['description']),
            ('enable_tap', True)
        ]

        expected_data = self._update_expected_response_data(
            data={
                'tap_enabled': True
            }
        )
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = (self.cmd.take_action(parsed_args))

        self.neutronclient.create_sfc_port_pair_group.assert_called_once_with({
            'port_pair_group': {
                'name': self._port_pair_group['name'],
                'port_pairs': [self._port_pair_group['port_pairs']],
                'description': self._port_pair_group['description'],
                'tap_enabled': True
            }
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(expected_data, data)

    def _update_expected_response_data(self, data):
        # REVISIT(vks1) - This method can be common for other test functions.
        ppg = fakes.FakeSfcPortPairGroup.create_port_pair_group(data)
        self.neutronclient.create_sfc_port_pair_group.return_value = {
            'port_pair_group': ppg}
        return self.get_data(ppg)


class TestDeleteSfcPortPairGroup(fakes.TestNeutronClientOSCV2):

    _port_pair_group = (fakes.FakeSfcPortPairGroup.create_port_pair_groups
                        (count=1))

    def setUp(self):
        super(TestDeleteSfcPortPairGroup, self).setUp()
        mock.patch(
            'neutronclient.osc.v2.sfc.sfc_port_pair_group._get_id',
            new=_get_id).start()
        self.neutronclient.delete_sfc_port_pair_group = mock.Mock(
            return_value=None)
        self.cmd = sfc_port_pair_group.DeleteSfcPortPairGroup(self.app,
                                                              self.namespace)

    def test_delete_port_pair_group(self):
        client = self.app.client_manager.neutronclient
        mock_port_pair_group_delete = client.delete_sfc_port_pair_group
        arglist = [
            self._port_pair_group[0]['id'],
        ]
        verifylist = [
            ('port_pair_group', self._port_pair_group[0]['id']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        mock_port_pair_group_delete.assert_called_once_with(
            self._port_pair_group[0]['id'])
        self.assertIsNone(result)


class TestListSfcPortPairGroup(fakes.TestNeutronClientOSCV2):
    _ppgs = fakes.FakeSfcPortPairGroup.create_port_pair_groups(count=1)
    columns = ('ID', 'Name', 'Port Pair', 'Port Pair Group Parameters',
               'Tap Enabled')
    columns_long = ('ID', 'Name', 'Port Pair', 'Port Pair Group Parameters',
                    'Description', 'Loadbalance ID', 'Project', 'Tap Enabled')
    _port_pair_group = _ppgs[0]
    data = [
        _port_pair_group['id'],
        _port_pair_group['name'],
        _port_pair_group['port_pairs'],
        _port_pair_group['port_pair_group_parameters'],
        _port_pair_group['tap_enabled']
    ]
    data_long = [
        _port_pair_group['id'],
        _port_pair_group['name'],
        _port_pair_group['port_pairs'],
        _port_pair_group['port_pair_group_parameters'],
        _port_pair_group['description'],
        _port_pair_group['tap_enabled']
    ]
    _port_pair_group1 = {'port_pair_groups': _port_pair_group}
    _port_pair_id = _port_pair_group['id']

    def setUp(self):
        super(TestListSfcPortPairGroup, self).setUp()
        mock.patch(
            'neutronclient.osc.v2.sfc.sfc_port_pair_group._get_id',
            new=_get_id).start()

        self.neutronclient.list_sfc_port_pair_groups = mock.Mock(
            return_value={'port_pair_groups': self._ppgs}
        )
        # Get the command object to test
        self.cmd = sfc_port_pair_group.ListSfcPortPairGroup(self.app,
                                                            self.namespace)

    def test_list_port_pair_groups(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns = self.cmd.take_action(parsed_args)[0]
        ppgs = self.neutronclient \
            .list_sfc_port_pair_groups()['port_pair_groups']
        ppg = ppgs[0]
        data = [
            ppg['id'],
            ppg['name'],
            ppg['port_pairs'],
            ppg['port_pair_group_parameters'],
            ppg['tap_enabled']
        ]
        self.assertEqual(list(self.columns), columns)
        self.assertEqual(self.data, data)

    def test_list_with_long_option(self):
        arglist = ['--long']
        verifylist = [('long', True)]
        ppgs = self.neutronclient \
            .list_sfc_port_pair_groups()['port_pair_groups']
        ppg = ppgs[0]
        data = [
            ppg['id'],
            ppg['name'],
            ppg['port_pairs'],
            ppg['port_pair_group_parameters'],
            ppg['description'],
            ppg['tap_enabled']
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns_long = self.cmd.take_action(parsed_args)[0]
        self.assertEqual(list(self.columns_long), columns_long)
        self.assertEqual(self.data_long, data)


class TestSetSfcPortPairGroup(fakes.TestNeutronClientOSCV2):
    _port_pair_group = fakes.FakeSfcPortPairGroup.create_port_pair_group()
    resource = _port_pair_group
    res = 'port_pair_group'
    _port_pair_group_name = _port_pair_group['name']
    ppg_pp = _port_pair_group['port_pairs']
    _port_pair_group_id = _port_pair_group['id']

    def setUp(self):
        super(TestSetSfcPortPairGroup, self).setUp()

        mock.patch(
            'neutronclient.osc.v2.sfc.sfc_port_pair_group._get_id',
            new=_get_id).start()
        self.neutronclient.update_sfc_port_pair_group = mock.Mock(
            return_value=None)
        self.mocked = self.neutronclient.update_sfc_port_pair_group
        self.cmd = sfc_port_pair_group.SetSfcPortPairGroup(self.app,
                                                           self.namespace)

    def test_set_port_pair_group(self):
        target = self.resource['id']
        port_pair1 = 'additional_port1'
        port_pair2 = 'additional_port2'

        def _mock_port_pair_group(*args, **kwargs):

            if self.neutronclient.find_resource.call_count == 1:
                self.neutronclient.find_resource.assert_called_with(
                    'port_pair', port_pair1, cmd_resource='sfc_port_pair')
                return {'id': args[1]}

            if self.neutronclient.find_resource.call_count == 2:
                self.neutronclient.find_resource.assert_called_with(
                    'port_pair', port_pair2, cmd_resource='sfc_port_pair')
                return {'id': args[1]}

            if self.neutronclient.find_resource.call_count == 3:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource='sfc_port_pair_group')
                return {'port_pairs': [self.ppg_pp]}

        self.neutronclient.find_resource.side_effect = _mock_port_pair_group

        arglist = [
            target,
            '--port-pair', port_pair1,
            '--port-pair', port_pair2,
        ]
        verifylist = [
            (self.res, target),
            ('port_pairs', [port_pair1, port_pair2])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        expect = {'port_pairs': sorted([self.ppg_pp, port_pair1, port_pair2])}
        self.mocked.assert_called_once_with(target, {self.res: expect})
        self.assertEqual(3, self.neutronclient.find_resource.call_count)
        self.assertIsNone(result)

    def test_set_no_port_pair(self):
        client = self.app.client_manager.neutronclient
        mock_port_pair_group_update = client.update_sfc_port_pair_group
        arglist = [
            self._port_pair_group_name,
            '--name', 'name_updated',
            '--description', 'desc_updated',
            '--no-port-pair',
        ]
        verifylist = [
            ('port_pair_group', self._port_pair_group_name),
            ('name', 'name_updated'),
            ('description', 'desc_updated'),
            ('no_port_pair', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {'port_pair_group': {'name': 'name_updated',
                                     'description': 'desc_updated',
                                     'port_pairs': []}}
        mock_port_pair_group_update.assert_called_once_with(
            self._port_pair_group_name, attrs)
        self.assertIsNone(result)


class TestShowSfcPortPairGroup(fakes.TestNeutronClientOSCV2):

    _ppg = fakes.FakeSfcPortPairGroup.create_port_pair_group()
    data = (
        _ppg['description'],
        _ppg['id'],
        _ppg['group_id'],
        _ppg['name'],
        _ppg['port_pairs'],
        _ppg['port_pair_group_parameters'],
        _ppg['project_id'],
        _ppg['tap_enabled'])
    _port_pair_group = {'port_pair_group': _ppg}
    _port_pair_group_id = _ppg['id']
    columns = (
        'Description',
        'ID',
        'Loadbalance ID',
        'Name',
        'Port Pair',
        'Port Pair Group Parameters',
        'Project',
        'Tap Enabled'
    )

    def setUp(self):
        super(TestShowSfcPortPairGroup, self).setUp()
        mock.patch(
            'neutronclient.osc.v2.sfc.sfc_port_pair_group._get_id',
            new=_get_id).start()

        self.neutronclient.show_sfc_port_pair_group = mock.Mock(
            return_value=self._port_pair_group
        )
        self.cmd = sfc_port_pair_group.ShowSfcPortPairGroup(self.app,
                                                            self.namespace)

    def test_show_port_pair_group(self):
        client = self.app.client_manager.neutronclient
        mock_port_pair_group_show = client.show_sfc_port_pair_group
        arglist = [
            self._port_pair_group_id,
        ]
        verifylist = [
            ('port_pair_group', self._port_pair_group_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        mock_port_pair_group_show.assert_called_once_with(
            self._port_pair_group_id)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUnsetSfcPortPairGroup(fakes.TestNeutronClientOSCV2):
    _port_pair_group = fakes.FakeSfcPortPairGroup.create_port_pair_group()
    resource = _port_pair_group
    res = 'port_pair_group'
    _port_pair_group_name = _port_pair_group['name']
    _port_pair_group_id = _port_pair_group['id']
    ppg_pp = _port_pair_group['port_pairs']

    def setUp(self):
        super(TestUnsetSfcPortPairGroup, self).setUp()
        mock.patch(
            'neutronclient.osc.v2.sfc.sfc_port_pair_group._get_id',
            new=_get_id).start()
        self.neutronclient.update_sfc_port_pair_group = mock.Mock(
            return_value=None)
        self.mocked = self.neutronclient.update_sfc_port_pair_group
        self.cmd = sfc_port_pair_group.UnsetSfcPortPairGroup(
            self.app, self.namespace)

    def test_unset_port_pair(self):
        target = self.resource['id']
        port_pair1 = 'additional_port1'
        port_pair2 = 'additional_port2'

        def _mock_port_pair(*args, **kwargs):

            if self.neutronclient.find_resource.call_count == 1:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource='sfc_port_pair_group')
                return {'port_pairs': [self.ppg_pp]}

            if self.neutronclient.find_resource.call_count == 2:
                self.neutronclient.find_resource.assert_called_with(
                    'port_pair', port_pair1, cmd_resource='sfc_port_pair')
                return {'id': args[1]}

            if self.neutronclient.find_resource.call_count == 3:
                self.neutronclient.find_resource.assert_called_with(
                    'port_pair', port_pair2, cmd_resource='sfc_port_pair')
                return {'id': args[1]}

            if self.neutronclient.find_resource.call_count == 4:
                self.neutronclient.find_resource.assert_called_with(
                    self.res, target, cmd_resource='sfc_port_pair_group')
                return {'id': args[1]}

        self.neutronclient.find_resource.side_effect = _mock_port_pair

        arglist = [
            target,
            '--port-pair', port_pair1,
            '--port-pair', port_pair2,
        ]
        verifylist = [
            (self.res, target),
            ('port_pairs', [port_pair1, port_pair2])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        expect = {'port_pairs': sorted([self.ppg_pp])}
        self.mocked.assert_called_once_with(target, {self.res: expect})
        self.assertIsNone(result)

    def test_unset_all_port_pair(self):
        client = self.app.client_manager.neutronclient
        mock_port_pair_group_update = client.update_sfc_port_pair_group
        arglist = [
            self._port_pair_group_name,
            '--all-port-pair',
        ]
        verifylist = [
            ('port_pair_group', self._port_pair_group_name),
            ('all_port_pair', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {'port_pair_group': {'port_pairs': []}}
        mock_port_pair_group_update.assert_called_once_with(
            self._port_pair_group_name, attrs)
        self.assertIsNone(result)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from osc_lib.command import command
from osc_lib import utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _
from neutronclient.osc import utils as nc_osc_utils
from neutronclient.osc.v2.dynamic_routing import constants


def _get_attrs(client_manager, parsed_args):
    attrs = {}
    if parsed_args.name is not None:
        attrs['name'] = str(parsed_args.name)
    if 'local_as' in parsed_args:
        attrs['local_as'] = parsed_args.local_as
    if 'ip_version' in parsed_args:
        attrs['ip_version'] = parsed_args.ip_version
    if parsed_args.advertise_tenant_networks:
        attrs['advertise_tenant_networks'] = True
    if parsed_args.no_advertise_tenant_networks:
        attrs['advertise_tenant_networks'] = False
    if parsed_args.advertise_floating_ip_host_routes:
        attrs['advertise_floating_ip_host_routes'] = True
    if parsed_args.no_advertise_floating_ip_host_routes:
        attrs['advertise_floating_ip_host_routes'] = False

    if 'project' in parsed_args and parsed_args.project is not None:
        identity_client = client_manager.identity
        project_id = nc_osc_utils.find_project(
            identity_client,
            parsed_args.project,
            parsed_args.project_domain,
        ).id
        attrs['tenant_id'] = project_id
    return attrs


def add_common_arguments(parser):
    parser.add_argument(
        '--advertise-floating-ip-host-routes',
        action='store_true',
        help=_("Enable the advertisement of floating IP host routes "
               "by the BGP speaker. (default)"))
    parser.add_argument(
        '--no-advertise-floating-ip-host-routes',
        action='store_true',
        help=_("Disable the advertisement of floating IP host routes "
               "by the BGP speaker."))
    parser.add_argument(
        '--advertise-tenant-networks',
        action='store_true',
        help=_("Enable the advertisement of tenant network routes "
               "by the BGP speaker. (default)"))
    parser.add_argument(
        '--no-advertise-tenant-networks',
        action='store_true',
        help=_("Disable the advertisement of tenant network routes "
               "by the BGP speaker."))


class AddNetworkToSpeaker(command.Command):
    _description = _("Add a network to a BGP speaker")

    def get_parser(self, prog_name):
        parser = super(AddNetworkToSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='<bgp-speaker>',
            help=_("BGP speaker (name or ID)"))
        parser.add_argument(
            'network',
            metavar='<network>',
            help=_("Network to add (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        net_id = client.find_resource('network',
                                      parsed_args.network)['id']
        client.add_network_to_bgp_speaker(speaker_id, {'network_id': net_id})


class AddPeerToSpeaker(command.Command):
    _description = _("Add a peer to a BGP speaker")

    def get_parser(self, prog_name):
        parser = super(AddPeerToSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='<bgp-speaker>',
            help=_("BGP speaker (name or ID)"))
        parser.add_argument(
            'bgp_peer',
            metavar='<bgp-peer>',
            help=_("BGP Peer to add (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        peer_id = client.find_resource(constants.BGP_PEER,
                                       parsed_args.bgp_peer)['id']
        client.add_peer_to_bgp_speaker(speaker_id, {'bgp_peer_id': peer_id})


class CreateBgpSpeaker(command.ShowOne):
    _description = _("Create a BGP speaker")

    def get_parser(self, prog_name):
        parser = super(CreateBgpSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_("Name of the BGP speaker to create"))
        parser.add_argument(
            '--local-as',
            metavar='<local-as>',
            required=True,
            help=_("Local AS number. (Integer in [%(min_val)s, %(max_val)s] "
                   "is allowed.)") % {'min_val': constants.MIN_AS_NUM,
                                      'max_val': constants.MAX_AS_NUM})
        parser.add_argument(
            '--ip-version',
            type=int, choices=[4, 6],
            default=4,
            help=_("IP version for the BGP speaker (default is 4)"))
        add_common_arguments(parser)
        nc_osc_utils.add_project_owner_option_to_parser(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_attrs(self.app.client_manager, parsed_args)
        body = {}
        body[constants.BGP_SPEAKER] = attrs
        obj = client.create_bgp_speaker(body)[constants.BGP_SPEAKER]
        columns, display_columns = column_util.get_columns(obj)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


class DeleteBgpSpeaker(command.Command):
    _description = _("Delete a BGP speaker")

    def get_parser(self, prog_name):
        parser = super(DeleteBgpSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar="<bgp-speaker>",
            help=_("BGP speaker to delete (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGP_SPEAKER,
                                  parsed_args.bgp_speaker)['id']
        client.delete_bgp_speaker(id)


class ListBgpSpeaker(command.Lister):
    _description = _("List BGP speakers")

    def get_parser(self, prog_name):
        parser = super(ListBgpSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            '--agent',
            metavar='<agent-id>',
            help=_("List BGP speakers hosted by an agent (ID only)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        if parsed_args.agent is not None:
            data = client.list_bgp_speaker_on_dragent(parsed_args.agent)
        else:
            data = client.list_bgp_speakers()

        headers = ('ID', 'Name', 'Local AS', 'IP Version')
        columns = ('id', 'name', 'local_as', 'ip_version')
        return (headers, (utils.get_dict_properties(s, columns)
                          for s in data[constants.BGP_SPEAKERS]))


class ListRoutesAdvertisedBySpeaker(command.Lister):
    _description = _("List routes advertised")

    def get_parser(self, prog_name):
        parser = super(ListRoutesAdvertisedBySpeaker,
                       self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='<bgp-speaker>',
            help=_("BGP speaker (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        data = client.list_route_advertised_from_bgp_speaker(speaker_id)
        headers = ('Destination', 'Nexthop')
        columns = ('destination', 'next_hop')
        return (headers, (utils.get_dict_properties(s, columns)
                          for s in data['advertised_routes']))


class RemoveNetworkFromSpeaker(command.Command):
    _description = _("Remove a network from a BGP speaker")

    def get_parser(self, prog_name):
        parser = super(RemoveNetworkFromSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='<bgp-speaker>',
            help=_("BGP speaker (name or ID)"))
        parser.add_argument(
            'network',
            metavar='<network>',
            help=_("Network to remove (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        net_id = client.find_resource('network',
                                      parsed_args.network)['id']
        client.remove_network_from_bgp_speaker(speaker_id,
                                               {'network_id': net_id})


class RemovePeerFromSpeaker(command.Command):
    _description = _("Remove a peer from a BGP speaker")

    def get_parser(self, prog_name):
        parser = super(RemovePeerFromSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar='<bgp-speaker>',
            help=_("BGP speaker (name or ID)"))
        parser.add_argument(
            'bgp_peer',
            metavar='<bgp-peer>',
            help=_("BGP Peer to remove (name or ID)"))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        speaker_id = client.find_resource(constants.BGP_SPEAKER,
                                          parsed_args.bgp_speaker)['id']
        peer_id = client.find_resource(constants.BGP_PEER,
                                       parsed_args.bgp_peer)['id']
        client.remove_peer_from_bgp_speaker(speaker_id,
                                            {'bgp_peer_id': peer_id})


class SetBgpSpeaker(command.Command):
    _description = _("Set BGP speaker properties")

    resource = constants.BGP_SPEAKER

    def get_parser(self, prog_name):
        parser = super(SetBgpSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar="<bgp-speaker>",
            help=_("BGP speaker to update (name or ID)")
        )
        parser.add_argument(
            '--name',
            help=_("New name for the BGP speaker"))
        add_common_arguments(parser)
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGP_SPEAKER,
                                  parsed_args.bgp_speaker)['id']
        attrs = _get_attrs(self.app.client_manager, parsed_args)
        body = {}
        body[constants.BGP_SPEAKER] = attrs
        client.update_bgp_speaker(id, body)


class ShowBgpSpeaker(command.ShowOne):
    _description = _("Show a BGP speaker")

    def get_parser(self, prog_name):
        parser = super(ShowBgpSpeaker, self).get_parser(prog_name)
        parser.add_argument(
            'bgp_speaker',
            metavar="<bgp-speaker>",
            help=_("BGP speaker to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        id = client.find_resource(constants.BGP_SPEAKER,
                                  parsed_args.bgp_speaker)['id']
        obj = client.show_bgp_speaker(id)[constants.BGP_SPEAKER]
        columns, display_columns = column_util.get_columns(obj)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data



# Copyright 2016 Huawei Technologies India Pvt. Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.common import exceptions
from neutronclient.neutron.v2_0.bgp import peer as bgp_peer
from neutronclient.neutron.v2_0.bgp import speaker as bgp_speaker
from neutronclient.tests.unit import test_cli20


class CLITestV20BGPPeerJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['bgp_peer']

    def test_create_bgp_peer_with_mandatory_params(self):
        # Create BGP peer with mandatory params.
        resource = 'bgp_peer'
        cmd = bgp_peer.CreatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        name = 'my-name'
        my_id = 'my-id'
        peerip = '1.1.1.1'
        remote_asnum = '1'
        args = [name,
                '--peer-ip', peerip,
                '--remote-as', remote_asnum, ]
        position_names = ['name', 'peer_ip', 'remote_as',
                          'auth_type']
        position_values = [name, peerip, remote_asnum, 'none']
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_bgp_peer_with_all_params(self):
        # Create BGP peer with all params.
        resource = 'bgp_peer'
        cmd = bgp_peer.CreatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        name = 'my-name'
        my_id = 'my-id'
        peerip = '1.1.1.1'
        remote_asnum = '65535'
        authType = 'md5'
        password = 'abc'
        args = [name,
                '--peer-ip', peerip,
                '--remote-as', remote_asnum,
                '--auth-type', authType,
                '--password', password]
        position_names = ['name', 'peer_ip', 'remote_as',
                          'auth_type', 'password']
        position_values = [name, peerip, remote_asnum, authType, password]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_bgp_peer_with_invalid_min_remote_asnum(self):
        # Create BGP peer with invalid minimum remote-asnum.
        resource = 'bgp_peer'
        cmd = bgp_peer.CreatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        name = 'my-name'
        my_id = 'my-id'
        peerip = '1.1.1.1'
        remote_asnum = '0'
        args = [name,
                '--peer-ip', peerip,
                '--remote-as', remote_asnum, ]
        position_names = ['name', 'peer_ip', 'remote_as', ]
        position_values = [name, peerip, remote_asnum, ]
        exc = self.assertRaises(exceptions.CommandError,
                                self._test_create_resource,
                                resource, cmd, name, my_id, args,
                                position_names, position_values)
        self.assertEqual('remote-as "0" should be an integer [%s:%s].' %
                         (bgp_speaker.MIN_AS_NUM, bgp_speaker.MAX_AS_NUM),
                         str(exc))

    def test_create_bgp_peer_with_invalid_max_remote_asnum(self):
        # Create BGP peer with invalid maximum remote-asnum.
        resource = 'bgp_peer'
        cmd = bgp_peer.CreatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        name = 'my-name'
        my_id = 'my-id'
        peerip = '1.1.1.1'
        remote_asnum = '4294967296'
        args = [name,
                '--peer-ip', peerip,
                '--remote-as', remote_asnum, ]
        position_names = ['name', 'peer_ip', 'remote_as',
                          'auth_type', 'password']
        position_values = [name, peerip, remote_asnum, 'none', '']
        exc = self.assertRaises(exceptions.CommandError,
                                self._test_create_resource,
                                resource, cmd, name, my_id, args,
                                position_names, position_values)
        self.assertEqual('remote-as "4294967296" should be an '
                         'integer [%s:%s].' %
                         (bgp_speaker.MIN_AS_NUM, bgp_speaker.MAX_AS_NUM),
                         str(exc))

    def test_create_authenticated_bgp_peer_without_authtype(self):
        # Create authenticated BGP peer without auth-type.
        resource = 'bgp_peer'
        cmd = bgp_peer.CreatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        name = 'my-name'
        my_id = 'my-id'
        peerip = '1.1.1.1'
        remote_asnum = '2048'
        password = 'abc'
        args = [name,
                '--peer-ip', peerip,
                '--remote-as', remote_asnum,
                '--password', password]
        position_names = ['name', 'peer_ip', 'remote_as', 'password']
        position_values = [name, peerip, remote_asnum, password]
        exc = self.assertRaises(exceptions.CommandError,
                                self._test_create_resource,
                                resource, cmd, name, my_id, args,
                                position_names, position_values)
        self.assertEqual('Must provide auth-type if password is specified.',
                         str(exc))

    def test_create_authenticated_bgp_peer_without_password(self):
        # Create authenticated BGP peer without password.
        resource = 'bgp_peer'
        cmd = bgp_peer.CreatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        name = 'my-name'
        my_id = 'my-id'
        peerip = '1.1.1.1'
        remote_asnum = '2048'
        authType = 'md5'
        args = [name,
                '--peer-ip', peerip,
                '--remote-as', remote_asnum,
                '--auth-type', authType]
        position_names = ['name', 'peer_ip', 'remote_as', 'auth_type']
        position_values = [name, peerip, remote_asnum, authType]
        exc = self.assertRaises(exceptions.CommandError,
                                self._test_create_resource,
                                resource, cmd, name, my_id, args,
                                position_names, position_values)
        self.assertEqual('Must provide password if auth-type is specified.',
                         str(exc))

    def test_update_bgp_peer(self):
        # Update BGP peer:
        # myid --advertise-tenant-networks True
        #      --advertise-floating-ip-host-routes False
        resource = 'bgp_peer'
        cmd = bgp_peer.UpdatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'new-name',
                                    '--password', 'abc'],
                                   {'name': 'new-name', 'password': 'abc'})

    def test_update_bgp_peer_exception(self):
        # Update BGP peer: myid.
        resource = 'bgp_peer'
        cmd = bgp_peer.UpdatePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        self.assertRaises(exceptions.CommandError,
                          self._test_update_resource,
                          resource, cmd, 'myid', ['myid'], {})

    def test_list_bgp_peer(self):
        # List all BGP peers.
        resources = "bgp_peers"
        cmd = bgp_peer.ListPeers(test_cli20.MyApp(sys.stdout),
                                 None)
        self._test_list_resources(resources, cmd, True)

    # TODO(Vikram): Add test_list_bgp_peer_pagination

    def test_list_bgp_peer_sort(self):
        # sorted list: bgp-peer-list --sort-key name --sort-key id
        #                            --sort-key asc --sort-key desc
        resources = "bgp_peers"
        cmd = bgp_peer.ListPeers(test_cli20.MyApp(sys.stdout),
                                 None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_bgp_peer_limit(self):
        # size (1000) limited list: bgp-peer-list -P.
        resources = "bgp_peers"
        cmd = bgp_peer.ListPeers(test_cli20.MyApp(sys.stdout),
                                 None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_bgp_peer(self):
        # Show BGP peer: --fields id --fields name myid.
        resource = 'bgp_peer'
        cmd = bgp_peer.ShowPeer(test_cli20.MyApp(sys.stdout),
                                None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])

    def test_delete_bgp_peer(self):
        # Delete BGP peer: bgp_peer_id.
        resource = 'bgp_peer'
        cmd = bgp_peer.DeletePeer(test_cli20.MyApp(sys.stdout),
                                  None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)



# Copyright 2015 Huawei Technologies India Pvt Ltd, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#


from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronv20
from neutronclient.neutron.v2_0.qos import policy as qos_policy


def add_policy_argument(parser):
    parser.add_argument(
        'policy', metavar='QOS_POLICY',
        help=_('ID or name of the QoS policy.'))


def add_rule_argument(parser):
    parser.add_argument(
        'rule', metavar='QOS_RULE',
        help=_('ID of the QoS rule.'))


def update_policy_args2body(parsed_args, body):
    neutronv20.update_dict(parsed_args, body, ['policy'])


def update_rule_args2body(parsed_args, body):
    neutronv20.update_dict(parsed_args, body, ['rule'])


class QosRuleMixin(object):
    def add_known_arguments(self, parser):
        add_policy_argument(parser)

    def set_extra_attrs(self, parsed_args):
        self.parent_id = qos_policy.get_qos_policy_id(self.get_client(),
                                                      parsed_args.policy)

    def args2body(self, parsed_args):
        body = {}
        update_policy_args2body(parsed_args, body)
        return {'qos_rule': body}


class ListQoSRuleTypes(neutronv20.ListCommand):
    """List available qos rule types."""

    resource = 'rule_type'
    shadow_resource = 'qos_rule_type'
    pagination_support = True
    sorting_support = True



# Copyright 2013 Big Switch Networks
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronv20


def add_common_args(parser):
    parser.add_argument(
        '--name',
        help=_('Name for the firewall.'))
    parser.add_argument(
        '--description',
        help=_('Description for the firewall.'))
    router = parser.add_mutually_exclusive_group()
    router.add_argument(
        '--router',
        dest='routers',
        metavar='ROUTER',
        action='append',
        help=_('ID or name of the router associated with the firewall '
               '(requires FWaaS router insertion extension to be enabled). '
               'This option can be repeated.'))
    router.add_argument(
        '--no-routers',
        action='store_true',
        help=_('Associate no routers with the firewall (requires FWaaS '
               'router insertion extension).'))


def parse_common_args(client, parsed_args):
    body = {}
    if parsed_args.policy:
        body['firewall_policy_id'] = neutronv20.find_resourceid_by_name_or_id(
            client, 'firewall_policy',
            parsed_args.policy)

    if parsed_args.routers:
        body['router_ids'] = [
            neutronv20.find_resourceid_by_name_or_id(client, 'router', r)
            for r in parsed_args.routers]
    elif parsed_args.no_routers:
        body['router_ids'] = []

    neutronv20.update_dict(parsed_args, body,
                           ['name', 'description'])
    return body


class ListFirewall(neutronv20.ListCommand):
    """List firewalls that belong to a given tenant."""

    resource = 'firewall'
    list_columns = ['id', 'name', 'firewall_policy_id']
    _formatters = {}
    pagination_support = True
    sorting_support = True


class ShowFirewall(neutronv20.ShowCommand):
    """Show information of a given firewall."""

    resource = 'firewall'


class CreateFirewall(neutronv20.CreateCommand):
    """Create a firewall."""

    resource = 'firewall'

    def add_known_arguments(self, parser):
        add_common_args(parser)
        parser.add_argument(
            'policy', metavar='POLICY',
            help=_('ID or name of the firewall policy '
                   'associated to this firewall.'))
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state',
            action='store_false',
            help=_('Set admin state up to false.'))

    def args2body(self, parsed_args):
        body = parse_common_args(self.get_client(), parsed_args)
        neutronv20.update_dict(parsed_args, body, ['tenant_id'])
        body['admin_state_up'] = parsed_args.admin_state
        return {self.resource: body}


class UpdateFirewall(neutronv20.UpdateCommand):
    """Update a given firewall."""

    resource = 'firewall'

    def add_known_arguments(self, parser):
        add_common_args(parser)
        parser.add_argument(
            '--policy', metavar='POLICY',
            help=_('ID or name of the firewall policy '
                   'associated to this firewall.'))
        utils.add_boolean_argument(
            parser, '--admin-state-up', dest='admin_state_up',
            help=_('Update the admin state for the firewall '
                   '(True means UP).'))

    def args2body(self, parsed_args):
        body = parse_common_args(self.get_client(), parsed_args)
        neutronv20.update_dict(parsed_args, body, ['admin_state_up'])
        return {self.resource: body}


class DeleteFirewall(neutronv20.DeleteCommand):
    """Delete a given firewall."""

    resource = 'firewall'



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from oslo_serialization import jsonutils

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def _format_allocation_pools(subnet):
    try:
        return '\n'.join([jsonutils.dumps(pool) for pool in
                          subnet['allocation_pools']])
    except (TypeError, KeyError):
        return ''


def _format_dns_nameservers(subnet):
    try:
        return '\n'.join([jsonutils.dumps(server) for server in
                          subnet['dns_nameservers']])
    except (TypeError, KeyError):
        return ''


def _format_host_routes(subnet):
    try:
        return '\n'.join([jsonutils.dumps(route) for route in
                          subnet['host_routes']])
    except (TypeError, KeyError):
        return ''


def add_updatable_arguments(parser):
    parser.add_argument(
        '--name',
        help=_('Name of this subnet.'))
    parser.add_argument(
        '--description',
        help=_('Description of this subnet.'))
    gateway_sg = parser.add_mutually_exclusive_group()
    gateway_sg.add_argument(
        '--gateway', metavar='GATEWAY_IP',
        help=_('Gateway IP of this subnet.'))
    gateway_sg.add_argument(
        '--no-gateway',
        action='store_true',
        help=_('Do not configure a gateway for this subnet.'))
    parser.add_argument(
        '--allocation-pool', metavar='start=IP_ADDR,end=IP_ADDR',
        action='append', dest='allocation_pools',
        type=utils.str2dict_type(required_keys=['start', 'end']),
        help=_('Allocation pool IP addresses for this subnet '
               '(This option can be repeated).'))
    parser.add_argument(
        '--allocation_pool',
        action='append', dest='allocation_pools',
        type=utils.str2dict_type(required_keys=['start', 'end']),
        help=argparse.SUPPRESS)
    parser.add_argument(
        '--host-route', metavar='destination=CIDR,nexthop=IP_ADDR',
        action='append', dest='host_routes',
        type=utils.str2dict_type(required_keys=['destination', 'nexthop']),
        help=_('Additional route (This option can be repeated).'))
    parser.add_argument(
        '--dns-nameserver', metavar='DNS_NAMESERVER',
        action='append', dest='dns_nameservers',
        help=_('DNS name server for this subnet '
               '(This option can be repeated).'))
    parser.add_argument(
        '--disable-dhcp',
        action='store_true',
        help=_('Disable DHCP for this subnet.'))
    parser.add_argument(
        '--enable-dhcp',
        action='store_true',
        help=_('Enable DHCP for this subnet.'))
    # NOTE(ihrachys): yes, that's awful, but should be left as-is for
    # backwards compatibility for versions <=2.3.4 that passed the
    # boolean values through to the server without any argument
    # validation.
    parser.add_argument(
        '--enable-dhcp=True',
        action='store_true',
        dest='enable_dhcp',
        help=argparse.SUPPRESS)
    parser.add_argument(
        '--enable-dhcp=False',
        action='store_true',
        dest='disable_dhcp',
        help=argparse.SUPPRESS)


def updatable_args2body(parsed_args, body, for_create=True, ip_version=None):
    if parsed_args.disable_dhcp and parsed_args.enable_dhcp:
        raise exceptions.CommandError(_(
            "You cannot enable and disable DHCP at the same time."))

    neutronV20.update_dict(parsed_args, body,
                           ['name', 'allocation_pools',
                            'host_routes', 'dns_nameservers',
                            'description'])
    if parsed_args.no_gateway:
        body['gateway_ip'] = None
    elif parsed_args.gateway:
        body['gateway_ip'] = parsed_args.gateway
    if parsed_args.disable_dhcp:
        body['enable_dhcp'] = False
    if parsed_args.enable_dhcp:
        body['enable_dhcp'] = True
    if for_create and parsed_args.ipv6_ra_mode:
        if ip_version == 4:
            raise exceptions.CommandError(_("--ipv6-ra-mode is invalid "
                                            "when --ip-version is 4"))
        body['ipv6_ra_mode'] = parsed_args.ipv6_ra_mode
    if for_create and parsed_args.ipv6_address_mode:
        if ip_version == 4:
            raise exceptions.CommandError(_("--ipv6-address-mode is "
                                            "invalid when --ip-version "
                                            "is 4"))
        body['ipv6_address_mode'] = parsed_args.ipv6_address_mode


class ListSubnet(neutronV20.ListCommand):
    """List subnets that belong to a given tenant."""

    resource = 'subnet'
    _formatters = {'allocation_pools': _format_allocation_pools,
                   'dns_nameservers': _format_dns_nameservers,
                   'host_routes': _format_host_routes, }
    list_columns = ['id', 'name', 'cidr', 'allocation_pools']
    pagination_support = True
    sorting_support = True


class ShowSubnet(neutronV20.ShowCommand):
    """Show information of a given subnet."""

    resource = 'subnet'


class CreateSubnet(neutronV20.CreateCommand):
    """Create a subnet for a given tenant."""

    resource = 'subnet'

    def add_known_arguments(self, parser):
        add_updatable_arguments(parser)
        parser.add_argument(
            '--ip-version',
            type=int,
            default=4, choices=[4, 6],
            help=_('IP version to use, default is 4. '
                   'Note that when subnetpool is specified, '
                   'IP version is determined from the subnetpool '
                   'and this option is ignored.'))
        parser.add_argument(
            '--ip_version',
            type=int,
            choices=[4, 6],
            help=argparse.SUPPRESS)
        parser.add_argument(
            'network_id', metavar='NETWORK',
            help=_('Network ID or name this subnet belongs to.'))
        parser.add_argument(
            'cidr', nargs='?', metavar='CIDR',
            help=_('CIDR of subnet to create.'))
        parser.add_argument(
            '--ipv6-ra-mode',
            type=utils.convert_to_lowercase,
            choices=['dhcpv6-stateful', 'dhcpv6-stateless', 'slaac'],
            help=_('IPv6 RA (Router Advertisement) mode.'))
        parser.add_argument(
            '--ipv6-address-mode',
            type=utils.convert_to_lowercase,
            choices=['dhcpv6-stateful', 'dhcpv6-stateless', 'slaac'],
            help=_('IPv6 address mode.'))
        parser.add_argument(
            '--subnetpool', metavar='SUBNETPOOL',
            help=_('ID or name of subnetpool from which this subnet '
                   'will obtain a CIDR.'))
        parser.add_argument(
            '--use-default-subnetpool',
            action='store_true',
            help=_('Use default subnetpool for ip_version, if it exists.'))
        parser.add_argument(
            '--prefixlen', metavar='PREFIX_LENGTH',
            help=_('Prefix length for subnet allocation from subnetpool.'))
        parser.add_argument(
            '--segment', metavar='SEGMENT',
            help=_('ID of segment with which this subnet will be associated.'))

    def args2body(self, parsed_args):
        _network_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'network', parsed_args.network_id)
        body = {'network_id': _network_id}

        if parsed_args.prefixlen:
            body['prefixlen'] = parsed_args.prefixlen
        ip_version = parsed_args.ip_version
        if parsed_args.use_default_subnetpool:
            body['use_default_subnetpool'] = True
        if parsed_args.segment:
            body['segment_id'] = neutronV20.find_resourceid_by_name_or_id(
                self.get_client(), 'segment', parsed_args.segment)
        if parsed_args.subnetpool:
            if parsed_args.subnetpool == 'None':
                _subnetpool_id = None
            else:
                _subnetpool = neutronV20.find_resource_by_name_or_id(
                    self.get_client(), 'subnetpool', parsed_args.subnetpool)
                _subnetpool_id = _subnetpool['id']
                # Now that we have the pool_id - let's just have a check on the
                # ip version used in the pool
                ip_version = _subnetpool['ip_version']
            body['subnetpool_id'] = _subnetpool_id

        # IP version needs to be set as IP version can be
        # determined by subnetpool.
        body['ip_version'] = ip_version

        if parsed_args.cidr:
            # With subnetpool, cidr is now optional for creating subnet.
            cidr = parsed_args.cidr
            body['cidr'] = cidr
            unusable_cidr = '/32' if ip_version == 4 else '/128'
            if cidr.endswith(unusable_cidr):
                self.log.warning(_("An IPv%(ip)d subnet with a %(cidr)s CIDR "
                                   "will have only one usable IP address so "
                                   "the device attached to it will not have "
                                   "any IP connectivity."),
                                 {"ip": ip_version,
                                  "cidr": unusable_cidr})

        updatable_args2body(parsed_args, body, ip_version=ip_version)
        if parsed_args.tenant_id:
            body['tenant_id'] = parsed_args.tenant_id

        return {'subnet': body}


class DeleteSubnet(neutronV20.DeleteCommand):
    """Delete a given subnet."""

    resource = 'subnet'


class UpdateSubnet(neutronV20.UpdateCommand):
    """Update subnet's information."""

    resource = 'subnet'

    def add_known_arguments(self, parser):
        add_updatable_arguments(parser)

    def args2body(self, parsed_args):
        body = {}
        updatable_args2body(parsed_args, body, for_create=False)
        return {'subnet': body}



# Copyright (C) 2013 eNovance SAS <licensing@enovance.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from neutronclient.neutron.v2_0 import metering
from neutronclient.tests.unit import test_cli20


class CLITestV20MeteringJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['metering_label', 'metering_label_rule']

    def test_create_metering_label(self):
        # Create a metering label.
        resource = 'metering_label'
        cmd = metering.CreateMeteringLabel(
            test_cli20.MyApp(sys.stdout), None)
        name = 'my label'
        myid = 'myid'
        description = 'my description'
        args = [name, '--description', description, '--shared']
        position_names = ['name', 'description', 'shared']
        position_values = [name, description, True]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_list_metering_labels(self):
        resources = "metering_labels"
        cmd = metering.ListMeteringLabel(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd)

    def test_delete_metering_label(self):
        # Delete a metering label.
        resource = 'metering_label'
        cmd = metering.DeleteMeteringLabel(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    def test_show_metering_label(self):
        resource = 'metering_label'
        cmd = metering.ShowMeteringLabel(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id'])

    def test_create_metering_label_rule(self):
        resource = 'metering_label_rule'
        cmd = metering.CreateMeteringLabelRule(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        metering_label_id = 'aaa'
        remote_ip_prefix = '10.0.0.0/24'
        direction = 'ingress'
        args = [metering_label_id, remote_ip_prefix, '--direction', direction,
                '--excluded']
        position_names = ['metering_label_id', 'remote_ip_prefix', 'direction',
                          'excluded']
        position_values = [metering_label_id, remote_ip_prefix,
                           direction, True]
        self._test_create_resource(resource, cmd, metering_label_id,
                                   myid, args, position_names, position_values)

    def test_list_metering_label_rules(self):
        resources = "metering_label_rules"
        cmd = metering.ListMeteringLabelRule(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd)

    def test_delete_metering_label_rule(self):
        resource = 'metering_label_rule'
        cmd = metering.DeleteMeteringLabelRule(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    def test_show_metering_label_rule(self):
        resource = 'metering_label_rule'
        cmd = metering.ShowMeteringLabelRule(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id'])



# Copyright 2015 Huawei Technologies India Pvt. Ltd.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.common import exceptions
from neutronclient.neutron.v2_0 import address_scope
from neutronclient.tests.unit import test_cli20


class CLITestV20AddressScopeJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['address_scope']

    def setUp(self):
        super(CLITestV20AddressScopeJSON, self).setUp(plurals={'tags': 'tag'})

    def test_create_address_scope_with_minimum_option_ipv4(self):
        """Create address_scope: foo-address-scope with minimum option."""
        resource = 'address_scope'
        cmd = address_scope.CreateAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        name = 'foo-address-scope'
        myid = 'myid'
        args = [name, '4']
        position_names = ['name', 'ip_version']
        position_values = [name, 4]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_address_scope_with_minimum_option_ipv6(self):
        """Create address_scope: foo-address-scope with minimum option."""
        resource = 'address_scope'
        cmd = address_scope.CreateAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        name = 'foo-address-scope'
        myid = 'myid'
        args = [name, '6']
        position_names = ['name', 'ip_version']
        position_values = [name, 6]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_address_scope_with_minimum_option_bad_ip_version(self):
        """Create address_scope: foo-address-scope with minimum option."""
        resource = 'address_scope'
        cmd = address_scope.CreateAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        name = 'foo-address-scope'
        myid = 'myid'
        args = [name, '5']
        position_names = ['name', 'ip_version']
        position_values = [name, 5]
        self.assertRaises(SystemExit, self._test_create_resource,
                          resource, cmd, name, myid, args, position_names,
                          position_values)

    def test_create_address_scope_with_all_option(self):
        # Create address_scope: foo-address-scope with all options.
        resource = 'address_scope'
        cmd = address_scope.CreateAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        name = 'foo-address-scope'
        myid = 'myid'
        args = [name, '4', '--shared']
        position_names = ['name', 'ip_version', 'shared']
        position_values = [name, 4, True]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_address_scope_with_unicode(self):
        # Create address_scope: u'\u7f51\u7edc'.
        resource = 'address_scope'
        cmd = address_scope.CreateAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        name = u'\u7f51\u7edc'
        ip_version = u'4'
        myid = 'myid'
        args = [name, ip_version]
        position_names = ['name', 'ip_version']
        position_values = [name, 4]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_update_address_scope_exception(self):
        # Update address_scope (Negative) : myid.
        resource = 'address_scope'
        cmd = address_scope.UpdateAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        self.assertRaises(exceptions.CommandError, self._test_update_resource,
                          resource, cmd, 'myid', ['myid'], {})

    def test_update_address_scope(self):
        # Update address_scope: myid --name newname-address-scope.
        resource = 'address_scope'
        cmd = address_scope.UpdateAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname-address-scope'],
                                   {'name': 'newname-address-scope'}
                                   )
        # Update address_scope: myid --shared
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--shared', 'True'],
                                   {'shared': "True"}
                                   )

    def test_list_address_scope(self):
        # address_scope-list.
        resources = "address_scopes"
        cmd = address_scope.ListAddressScope(test_cli20.MyApp(sys.stdout),
                                             None)
        self._test_list_resources(resources, cmd, True)

    @mock.patch.object(address_scope.ListAddressScope, "extend_list")
    def test_list_address_scope_pagination(self, mock_extend_list):
        # address_scope-list.
        cmd = address_scope.ListAddressScope(test_cli20.MyApp(sys.stdout),
                                             None)
        self._test_list_resources_with_pagination("address_scopes",
                                                  cmd)
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    def test_list_address_scope_sort(self):
        # sorted list:
        # address_scope-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "address_scopes"
        cmd = address_scope.ListAddressScope(test_cli20.MyApp(sys.stdout),
                                             None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id", "ip_version"],
                                  sort_dir=["asc", "desc"])

    def test_list_address_scope_limit(self):
        # size (1000) limited list: address_scope-list -P.
        resources = "address_scopes"
        cmd = address_scope.ListAddressScope(test_cli20.MyApp(sys.stdout),
                                             None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_address_scope(self):
        # Show address_scope: --fields id --fields name myid.
        resource = 'address_scope'
        cmd = address_scope.ShowAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id,
                '--fields', 'ip_version', '6']
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name', 'ip_version'])

    def test_delete_address_scope(self):
        # Delete address_scope: address_scope_id.
        resource = 'address_scope'
        cmd = address_scope.DeleteAddressScope(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import os_client_config
from tempest.lib.cli import base


def credentials(cloud='devstack-admin'):
    """Retrieves credentials to run functional tests

    Credentials are either read via os-client-config from the environment
    or from a config file ('clouds.yaml'). Environment variables override
    those from the config file.

    devstack produces a clouds.yaml with two named clouds - one named
    'devstack' which has user privs and one named 'devstack-admin' which
    has admin privs. This function will default to getting the devstack-admin
    cloud as that is the current expected behavior.
    """
    return get_cloud_config(cloud=cloud).get_auth_args()


def get_cloud_config(cloud='devstack-admin'):
    return os_client_config.OpenStackConfig().get_one_cloud(cloud=cloud)


class ClientTestBase(base.ClientTestBase):
    """This is a first pass at a simple read only python-neutronclient test.

    This only exercises client commands that are read only.
    This should test commands:
    * as a regular user
    * as an admin user
    * with and without optional parameters
    * initially just check return codes, and later test command outputs

    """

    def _get_clients_from_os_cloud_config(self, cloud='devstack-admin'):
        creds = credentials(cloud)
        cli_dir = os.environ.get(
            'OS_NEUTRONCLIENT_EXEC_DIR',
            os.path.join(os.path.abspath('.'), '.tox/functional/bin'))

        return base.CLIClient(
            username=creds['username'],
            password=creds['password'],
            tenant_name=creds['project_name'],
            project_domain_id=creds['project_domain_id'],
            user_domain_id=creds['user_domain_id'],
            uri=creds['auth_url'],
            cli_dir=cli_dir)

    def _get_clients(self):
        return self._get_clients_from_os_cloud_config()

    def neutron(self, *args, **kwargs):
        return self.clients.neutron(*args, **kwargs)

    def neutron_non_admin(self, *args, **kwargs):
        if not hasattr(self, '_non_admin_clients'):
            self._non_admin_clients = self._get_clients_from_os_cloud_config(
                cloud='devstack')
        return self._non_admin_clients.neutron(*args, **kwargs)

    def is_extension_enabled(self, extension_alias):
        extensions = self.parser.listing(self.neutron('ext-list'))
        aliases = [e['alias'] for e in extensions]
        return extension_alias in aliases



#!/usr/bin/env python
# Copyright 2012 Red Hat
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
from unittest import mock
import urllib.parse as urlparse

from oslo_utils import uuidutils

from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron.v2_0 import securitygroup
from neutronclient.tests.unit import test_cli20


class CLITestV20SecurityGroupsJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['security_group', 'security_group_rule']

    def test_create_security_group(self):
        # Create security group: webservers.
        resource = 'security_group'
        cmd = securitygroup.CreateSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        name = 'webservers'
        myid = 'myid'
        args = [name, ]
        position_names = ['name']
        position_values = [name]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_security_group_tenant(self):
        # Create security group: webservers.
        resource = 'security_group'
        cmd = securitygroup.CreateSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        name = 'webservers'
        description = 'my webservers'
        myid = 'myid'
        args = ['--tenant_id', 'tenant_id', '--description', description, name]
        position_names = ['name', 'description']
        position_values = [name, description]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenant_id')

    def test_create_security_group_with_description(self):
        # Create security group: webservers.
        resource = 'security_group'
        cmd = securitygroup.CreateSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        name = 'webservers'
        description = 'my webservers'
        myid = 'myid'
        args = [name, '--description', description]
        position_names = ['name', 'description']
        position_values = [name, description]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_list_security_groups(self):
        resources = "security_groups"
        cmd = securitygroup.ListSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_security_groups_pagination(self):
        resources = "security_groups"
        cmd = securitygroup.ListSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_security_groups_sort(self):
        resources = "security_groups"
        cmd = securitygroup.ListSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_security_groups_limit(self):
        resources = "security_groups"
        cmd = securitygroup.ListSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_security_group_id(self):
        resource = 'security_group'
        cmd = securitygroup.ShowSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id'])

    def test_show_security_group_id_name(self):
        resource = 'security_group'
        cmd = securitygroup.ShowSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_delete_security_group(self):
        # Delete security group: myid.
        resource = 'security_group'
        cmd = securitygroup.DeleteSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    def test_update_security_group(self):
        # Update security group: myid --name myname --description desc.
        resource = 'security_group'
        cmd = securitygroup.UpdateSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--description', 'mydescription'],
                                   {'name': 'myname',
                                    'description': 'mydescription'}
                                   )

    def test_update_security_group_with_unicode(self):
        resource = 'security_group'
        cmd = securitygroup.UpdateSecurityGroup(
            test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', u'\u7f51\u7edc',
                                    '--description', u'\u7f51\u7edc'],
                                   {'name': u'\u7f51\u7edc',
                                    'description': u'\u7f51\u7edc'}
                                   )

    def test_create_security_group_rule_full(self):
        # Create security group rule.
        resource = 'security_group_rule'
        cmd = securitygroup.CreateSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        direction = 'ingress'
        ethertype = 'IPv4'
        protocol = 'tcp'
        port_range_min = '22'
        port_range_max = '22'
        remote_ip_prefix = '10.0.0.0/24'
        security_group_id = '1'
        remote_group_id = '1'
        args = ['--remote_ip_prefix', remote_ip_prefix, '--direction',
                direction, '--ethertype', ethertype, '--protocol', protocol,
                '--port_range_min', port_range_min, '--port_range_max',
                port_range_max, '--remote_group_id', remote_group_id,
                security_group_id, '--description', 'PCI policy 1421912']
        position_names = ['remote_ip_prefix', 'direction', 'ethertype',
                          'protocol', 'port_range_min', 'port_range_max',
                          'remote_group_id', 'security_group_id']
        position_values = [remote_ip_prefix, direction, ethertype, protocol,
                           port_range_min, port_range_max, remote_group_id,
                           security_group_id]
        self._test_create_resource(resource, cmd, None, myid, args,
                                   position_names, position_values,
                                   description='PCI policy 1421912')

    def test_create_security_group_rule_with_integer_protocol_value(self):
        resource = 'security_group_rule'
        cmd = securitygroup.CreateSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        direction = 'ingress'
        ethertype = 'IPv4'
        protocol = '2'
        port_range_min = '22'
        port_range_max = '22'
        remote_ip_prefix = '10.0.0.0/24'
        security_group_id = '1'
        remote_group_id = '1'
        args = ['--remote_ip_prefix', remote_ip_prefix, '--direction',
                direction, '--ethertype', ethertype, '--protocol', protocol,
                '--port_range_min', port_range_min, '--port_range_max',
                port_range_max, '--remote_group_id', remote_group_id,
                security_group_id]
        position_names = ['remote_ip_prefix', 'direction', 'ethertype',
                          'protocol', 'port_range_min', 'port_range_max',
                          'remote_group_id', 'security_group_id']
        position_values = [remote_ip_prefix, direction, ethertype, protocol,
                           port_range_min, port_range_max, remote_group_id,
                           security_group_id]
        self._test_create_resource(resource, cmd, None, myid, args,
                                   position_names, position_values)

    def test_delete_security_group_rule(self):
        # Delete security group rule: myid.
        resource = 'security_group_rule'
        cmd = securitygroup.DeleteSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    @mock.patch.object(securitygroup.ListSecurityGroupRule, "extend_list")
    def test_list_security_group_rules(self, mock_extend_list):
        resources = "security_group_rules"
        cmd = securitygroup.ListSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    def _build_test_data(self, data, excess=0):
        # Length of a query filter on security group rule id
        # in these testcases, id='secgroupid%02d' (with len(id)=12)
        sec_group_id_filter_len = 12

        response = []
        replace_rules = {'security_group_id': 'security_group',
                         'remote_group_id': 'remote_group'}

        search_opts = {'fields': ['id', 'name']}
        sec_group_ids = set()
        for rule in data:
            for key in replace_rules:
                if rule.get(key):
                    sec_group_ids.add(rule[key])
                    response.append({'id': rule[key], 'name': 'default'})
        sec_group_ids = list(sec_group_ids)

        result = []

        sec_group_count = len(sec_group_ids)
        max_size = ((sec_group_id_filter_len * sec_group_count) - excess)
        chunk_size = max_size // sec_group_id_filter_len

        for i in range(0, sec_group_count, chunk_size):
            search_opts['id'] = sec_group_ids[i: i + chunk_size]
            params = utils.safe_encode_dict(search_opts)
            resp_str = self.client.serialize({'security_groups': response})

            result.append({
                'filter': urlparse.urlencode(params, doseq=1),
                'response': (test_cli20.MyResp(200), resp_str),
            })

        return result

    def test_extend_list(self):
        data = [{'name': 'default',
                 'remote_group_id': 'remgroupid%02d' % i}
                for i in range(10)]
        data.append({'name': 'default', 'remote_group_id': None})
        resources = "security_groups"

        cmd = securitygroup.ListSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)

        path = getattr(self.client, resources + '_path')
        responses = self._build_test_data(data)
        known_args, _vs = cmd.get_parser(
            'list' + resources).parse_known_args()
        resp = responses[0]['response']

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=resp) as mock_request:
            cmd.extend_list(data, known_args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(test_cli20.end_url(
                path, responses[0]['filter']), self.client),
            'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

    def test_extend_list_exceed_max_uri_len(self):
        data = [{'name': 'default',
                 'security_group_id': 'secgroupid%02d' % i,
                 'remote_group_id': 'remgroupid%02d' % i}
                for i in range(10)]
        data.append({'name': 'default',
                     'security_group_id': 'secgroupid10',
                     'remote_group_id': None})
        resources = "security_groups"

        cmd = securitygroup.ListSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        path = getattr(self.client, resources + '_path')
        responses = self._build_test_data(data, excess=1)

        known_args, _vs = cmd.get_parser(
            'list' + resources).parse_known_args()
        mock_request_side_effects = []
        mock_request_calls = []
        mock_check_uri_side_effects = [exceptions.RequestURITooLong(excess=1)]
        mock_check_uri_calls = [mock.call(mock.ANY)]
        for item in responses:
            mock_request_side_effects.append(item['response'])
            mock_request_calls.append(mock.call(
                test_cli20.MyUrlComparator(
                    test_cli20.end_url(path, item['filter']), self.client),
                'GET',
                body=None,
                headers=test_cli20.ContainsKeyValue(
                    {'X-Auth-Token': test_cli20.TOKEN})))
            mock_check_uri_side_effects.append(None)
            mock_check_uri_calls.append(mock.call(mock.ANY))

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient,
                                  "request") as mock_request, \
                mock.patch.object(self.client.httpclient,
                                  "_check_uri_length") as mock_check_uri:
            mock_request.side_effect = mock_request_side_effects
            mock_check_uri.side_effect = mock_check_uri_side_effects
            cmd.extend_list(data, known_args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_has_calls(mock_request_calls)
        mock_check_uri.assert_has_calls(mock_check_uri_calls)
        self.assertEqual(len(mock_request_calls), mock_request.call_count)
        self.assertEqual(len(mock_check_uri_calls), mock_check_uri.call_count)

    @mock.patch.object(securitygroup.ListSecurityGroupRule, "extend_list")
    def test_list_security_group_rules_pagination(self, mock_extend_list):
        resources = "security_group_rules"
        cmd = securitygroup.ListSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    @mock.patch.object(securitygroup.ListSecurityGroupRule, "extend_list")
    def test_list_security_group_rules_sort(self, mock_extend_list):
        resources = "security_group_rules"
        cmd = securitygroup.ListSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    @mock.patch.object(securitygroup.ListSecurityGroupRule, "extend_list")
    def test_list_security_group_rules_limit(self, mock_extend_list):
        resources = "security_group_rules"
        cmd = securitygroup.ListSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    def test_show_security_group_rule(self):
        resource = 'security_group_rule'
        cmd = securitygroup.ShowSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id'])

    def _test_list_security_group_rules_extend(self, api_data, expected,
                                               args=(), conv=True,
                                               query_fields=None):
        def setup_list_stub(resources, data, query, mock_calls, mock_returns):
            reses = {resources: data}
            resstr = self.client.serialize(reses)
            resp = (test_cli20.MyResp(200), resstr)
            path = getattr(self.client, resources + '_path')
            mock_calls.append(mock.call(
                test_cli20.MyUrlComparator(
                    test_cli20.end_url(path, query),
                    self.client),
                'GET',
                body=None,
                headers=test_cli20.ContainsKeyValue(
                    {'X-Auth-Token': test_cli20.TOKEN})))
            mock_returns.append(resp)

        cmd = securitygroup.ListSecurityGroupRule(
            test_cli20.MyApp(sys.stdout), None)
        query = ''
        if query_fields:
            query = '&'.join(['fields=' + f for f in query_fields])
        mock_request_calls = []
        mock_request_returns = []
        setup_list_stub('security_group_rules', api_data, query,
                        mock_request_calls, mock_request_returns)
        if conv:
            sec_ids = set()
            for n in api_data:
                sec_ids.add(n['security_group_id'])
                if n.get('remote_group_id'):
                    sec_ids.add(n['remote_group_id'])
            filters = ''
            for id in sec_ids:
                filters = filters + "&id=%s" % id
            setup_list_stub('security_groups',
                            [{'id': 'myid1', 'name': 'group1'},
                             {'id': 'myid2', 'name': 'group2'},
                             {'id': 'myid3', 'name': 'group3'}],
                            'fields=id&fields=name' + filters,
                            mock_request_calls,
                            mock_request_returns)

        cmd_parser = cmd.get_parser('list_security_group_rules')
        parsed_args = cmd_parser.parse_args(args)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient,
                                  "request") as mock_request:
            mock_request.side_effect = mock_request_returns
            result = cmd.take_action(parsed_args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), None)
        mock_request.assert_has_calls(mock_request_calls)
        self.assertEqual(len(mock_request_calls), mock_request.call_count)
        self.assertEqual(expected['cols'], result[0])
        # Check data
        _result = [x for x in result[1]]
        self.assertEqual(len(expected['data']), len(_result))
        for res, exp in zip(_result, expected['data']):
            self.assertEqual(len(exp), len(res))
            self.assertEqual(exp, res)

    def _test_list_security_group_rules_extend_sg_name(
            self, expected_mode=None, args=(), conv=True, query_field=False):
        if query_field:
            field_filters = ['id', 'security_group_id',
                             'remote_ip_prefix', 'remote_group_id']
        else:
            field_filters = None

        data = [self._prepare_rule(rule_id='ruleid1', sg_id='myid1',
                                   remote_group_id='myid1',
                                   filters=field_filters),
                self._prepare_rule(rule_id='ruleid2', sg_id='myid2',
                                   remote_group_id='myid3',
                                   filters=field_filters),
                self._prepare_rule(rule_id='ruleid3', sg_id='myid2',
                                   remote_group_id='myid2',
                                   filters=field_filters),
                ]

        if expected_mode == 'noconv':
            expected = {'cols': ['id', 'security_group_id', 'remote_group_id'],
                        'data': [('ruleid1', 'myid1', 'myid1'),
                                 ('ruleid2', 'myid2', 'myid3'),
                                 ('ruleid3', 'myid2', 'myid2')]}
        elif expected_mode == 'remote_group_id':
            expected = {'cols': ['id', 'security_group', 'remote_group'],
                        'data': [('ruleid1', 'group1', 'group1'),
                                 ('ruleid2', 'group2', 'group3'),
                                 ('ruleid3', 'group2', 'group2')]}
        else:
            expected = {'cols': ['id', 'security_group', 'remote'],
                        'data': [('ruleid1', 'group1', 'group1 (group)'),
                                 ('ruleid2', 'group2', 'group3 (group)'),
                                 ('ruleid3', 'group2', 'group2 (group)')]}

        self._test_list_security_group_rules_extend(
            data, expected, args=args, conv=conv, query_fields=field_filters)

    def test_list_security_group_rules_extend_remote_sg_name(self):
        args = '-c id -c security_group -c remote'.split()
        self._test_list_security_group_rules_extend_sg_name(args=args)

    def test_list_security_group_rules_extend_sg_name_noconv(self):
        args = '--no-nameconv -c id -c security_group_id -c remote_group_id'
        args = args.split()
        self._test_list_security_group_rules_extend_sg_name(
            expected_mode='noconv', args=args, conv=False)

    def test_list_security_group_rules_extend_sg_name_with_columns(self):
        args = '-c id -c security_group_id -c remote_group_id'.split()
        self._test_list_security_group_rules_extend_sg_name(
            expected_mode='remote_group_id', args=args)

    def test_list_security_group_rules_extend_sg_name_with_columns_no_id(self):
        args = '-c id -c security_group -c remote_group'.split()
        self._test_list_security_group_rules_extend_sg_name(
            expected_mode='remote_group_id', args=args)

    def test_list_security_group_rules_extend_sg_name_with_fields(self):
        # NOTE: remote_ip_prefix is required to show "remote" column
        args = ('-F id -F security_group_id '
                '-F remote_ip_prefix -F remote_group_id').split()
        self._test_list_security_group_rules_extend_sg_name(
            args=args, query_field=True)

    def test_list_security_group_rules_extend_sg_name_with_fields_no_id(self):
        # NOTE: remote_ip_prefix is required to show "remote" column
        args = ('-F id -F security_group '
                '-F remote_ip_prefix -F remote_group').split()
        self._test_list_security_group_rules_extend_sg_name(args=args,
                                                            query_field=True)

    def test_list_security_group_rules_extend_remote(self):
        args = '-c id -c security_group -c remote'.split()

        data = [self._prepare_rule(rule_id='ruleid1', sg_id='myid1',
                                   remote_ip_prefix='172.16.18.0/24'),
                self._prepare_rule(rule_id='ruleid2', sg_id='myid2',
                                   remote_ip_prefix='172.16.20.0/24'),
                self._prepare_rule(rule_id='ruleid3', sg_id='myid2',
                                   remote_group_id='myid3')]
        expected = {'cols': ['id', 'security_group', 'remote'],
                    'data': [('ruleid1', 'group1', '172.16.18.0/24 (CIDR)'),
                             ('ruleid2', 'group2', '172.16.20.0/24 (CIDR)'),
                             ('ruleid3', 'group2', 'group3 (group)')]}
        self._test_list_security_group_rules_extend(data, expected, args)

    def test_list_security_group_rules_extend_proto_port(self):
        data = [self._prepare_rule(rule_id='ruleid1', sg_id='myid1',
                                   protocol='tcp',
                                   port_range_min=22, port_range_max=22),
                self._prepare_rule(rule_id='ruleid2', sg_id='myid2',
                                   direction='egress', ethertype='IPv6',
                                   protocol='udp',
                                   port_range_min=80, port_range_max=81),
                self._prepare_rule(rule_id='ruleid3', sg_id='myid2',
                                   protocol='icmp',
                                   remote_ip_prefix='10.2.0.0/16')]
        expected = {
            'cols': ['id', 'security_group', 'direction', 'ethertype',
                     'port/protocol', 'remote'],
            'data': [
                ('ruleid1', 'group1', 'ingress', 'IPv4', '22/tcp', 'any'),
                ('ruleid2', 'group2', 'egress', 'IPv6', '80-81/udp', 'any'),
                ('ruleid3', 'group2', 'ingress', 'IPv4', 'icmp',
                 '10.2.0.0/16 (CIDR)')
            ]}
        self._test_list_security_group_rules_extend(data, expected)

    def _prepare_rule(self, rule_id=None, sg_id=None, tenant_id=None,
                      direction=None, ethertype=None,
                      protocol=None, port_range_min=None, port_range_max=None,
                      remote_ip_prefix=None, remote_group_id=None,
                      filters=None):
        rule = {'id': rule_id or uuidutils.generate_uuid(),
                'tenant_id': tenant_id or uuidutils.generate_uuid(),
                'security_group_id': sg_id or uuidutils.generate_uuid(),
                'direction': direction or 'ingress',
                'ethertype': ethertype or 'IPv4',
                'protocol': protocol,
                'port_range_min': port_range_min,
                'port_range_max': port_range_max,
                'remote_ip_prefix': remote_ip_prefix,
                'remote_group_id': remote_group_id}
        if filters:
            return dict([(k, v) for k, v in rule.items() if k in filters])
        else:
            return rule

    def test__get_remote_both_unspecified(self):
        sg_rule = self._prepare_rule(remote_ip_prefix=None,
                                     remote_group_id=None)
        self.assertIsNone(securitygroup._get_remote(sg_rule))

    def test__get_remote_remote_ip_prefix_specified(self):
        sg_rule = self._prepare_rule(remote_ip_prefix='172.16.18.0/24')
        self.assertEqual('172.16.18.0/24 (CIDR)',
                         securitygroup._get_remote(sg_rule))

    def test__get_remote_remote_group_specified(self):
        sg_rule = self._prepare_rule(remote_group_id='sg_id1')
        self.assertEqual('sg_id1 (group)', securitygroup._get_remote(sg_rule))

    def test__get_protocol_port_all_none(self):
        sg_rule = self._prepare_rule()
        self.assertIsNone(securitygroup._get_protocol_port(sg_rule))

    def test__get_protocol_port_tcp_all_port(self):
        sg_rule = self._prepare_rule(protocol='tcp')
        self.assertEqual('tcp', securitygroup._get_protocol_port(sg_rule))

    def test__get_protocol_port_tcp_one_port(self):
        sg_rule = self._prepare_rule(protocol='tcp',
                                     port_range_min=22, port_range_max=22)
        self.assertEqual('22/tcp', securitygroup._get_protocol_port(sg_rule))

    def test__get_protocol_port_tcp_port_range(self):
        sg_rule = self._prepare_rule(protocol='tcp',
                                     port_range_min=5000, port_range_max=5010)
        self.assertEqual('5000-5010/tcp',
                         securitygroup._get_protocol_port(sg_rule))

    def test__get_protocol_port_udp_all_port(self):
        sg_rule = self._prepare_rule(protocol='udp')
        self.assertEqual('udp', securitygroup._get_protocol_port(sg_rule))

    def test__get_protocol_port_udp_one_port(self):
        sg_rule = self._prepare_rule(protocol='udp',
                                     port_range_min=22, port_range_max=22)
        self.assertEqual('22/udp', securitygroup._get_protocol_port(sg_rule))

    def test__get_protocol_port_udp_port_range(self):
        sg_rule = self._prepare_rule(protocol='udp',
                                     port_range_min=5000, port_range_max=5010)
        self.assertEqual('5000-5010/udp',
                         securitygroup._get_protocol_port(sg_rule))

    def test__get_protocol_port_icmp_all(self):
        sg_rule = self._prepare_rule(protocol='icmp')
        self.assertEqual('icmp', securitygroup._get_protocol_port(sg_rule))

    def test_get_ethertype_for_protocol_icmpv6(self):
        self.assertEqual('IPv6',
                         securitygroup.generate_default_ethertype('icmpv6'))

    def test_get_ethertype_for_protocol_icmp(self):
        self.assertEqual('IPv4',
                         securitygroup.generate_default_ethertype('icmp'))

    def test__get_protocol_port_udp_code_type(self):
        sg_rule = self._prepare_rule(protocol='icmp',
                                     port_range_min=1, port_range_max=8)
        self.assertEqual('icmp (type:1, code:8)',
                         securitygroup._get_protocol_port(sg_rule))

    def test__format_sg_rules(self):
        rules = [self._prepare_rule(),
                 self._prepare_rule(protocol='tcp', port_range_min=80,
                                    port_range_max=80),
                 self._prepare_rule(remote_ip_prefix='192.168.1.0/24'),
                 self._prepare_rule(remote_group_id='group1'),
                 self._prepare_rule(protocol='tcp',
                                    remote_ip_prefix='10.1.1.0/24'),
                 self._prepare_rule(direction='egress'),
                 self._prepare_rule(direction='egress', ethertype='IPv6'),
                 ]
        sg = {'security_group_rules': rules}
        expected_data = ['ingress, IPv4',
                         'ingress, IPv4, 80/tcp',
                         'ingress, IPv4, remote_ip_prefix: 192.168.1.0/24',
                         'ingress, IPv4, remote_group_id: group1',
                         'ingress, IPv4, tcp, remote_ip_prefix: 10.1.1.0/24',
                         'egress, IPv4',
                         'egress, IPv6',
                         ]
        expected = '\n'.join(sorted(expected_data))
        self.assertEqual(expected, securitygroup._format_sg_rules(sg))



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def _format_prefixes(subnetpool):
    try:
        return '\n'.join(pool for pool in subnetpool['prefixes'])
    except (TypeError, KeyError):
        return subnetpool['prefixes']


def add_updatable_arguments(parser, for_create=False):
    parser.add_argument(
        '--description',
        help=_('Description of subnetpool.'))
    parser.add_argument(
        '--min-prefixlen', type=int,
        help=_('Subnetpool minimum prefix length.'))
    parser.add_argument(
        '--max-prefixlen', type=int,
        help=_('Subnetpool maximum prefix length.'))
    parser.add_argument(
        '--default-prefixlen', type=int,
        help=_('Subnetpool default prefix length.'))
    parser.add_argument(
        '--pool-prefix',
        action='append', dest='prefixes',
        required=for_create,
        help=_('Subnetpool prefixes (This option can be repeated).'))
    utils.add_boolean_argument(
        parser, '--is-default',
        help=_('Specify whether this should be the default subnetpool '
               '(True meaning default).'))


def updatable_args2body(parsed_args, body):
    neutronV20.update_dict(parsed_args, body,
                           ['name', 'prefixes', 'default_prefixlen',
                            'min_prefixlen', 'max_prefixlen', 'is_default',
                            'description'])


class ListSubnetPool(neutronV20.ListCommand):
    """List subnetpools that belong to a given tenant."""

    _formatters = {'prefixes': _format_prefixes, }
    resource = 'subnetpool'
    list_columns = ['id', 'name', 'prefixes',
                    'default_prefixlen', 'address_scope_id', 'is_default']
    pagination_support = True
    sorting_support = True


class ShowSubnetPool(neutronV20.ShowCommand):
    """Show information of a given subnetpool."""

    resource = 'subnetpool'


class CreateSubnetPool(neutronV20.CreateCommand):
    """Create a subnetpool for a given tenant."""

    resource = 'subnetpool'

    def add_known_arguments(self, parser):
        add_updatable_arguments(parser, for_create=True)
        parser.add_argument(
            '--shared',
            action='store_true',
            help=_('Set the subnetpool as shared.'))
        parser.add_argument(
            'name',
            metavar='NAME',
            help=_('Name of the subnetpool to be created.'))
        parser.add_argument(
            '--address-scope',
            metavar='ADDRSCOPE',
            help=_('ID or name of the address scope with which the subnetpool '
                   'is associated. Prefixes must be unique across address '
                   'scopes.'))

    def args2body(self, parsed_args):
        body = {'prefixes': parsed_args.prefixes}
        updatable_args2body(parsed_args, body)
        neutronV20.update_dict(parsed_args, body, ['tenant_id'])
        if parsed_args.shared:
            body['shared'] = True

        # Parse and update for "address-scope" option
        if parsed_args.address_scope:
            _addrscope_id = neutronV20.find_resourceid_by_name_or_id(
                self.get_client(), 'address_scope',
                parsed_args.address_scope)
            body['address_scope_id'] = _addrscope_id
        return {'subnetpool': body}


class DeleteSubnetPool(neutronV20.DeleteCommand):
    """Delete a given subnetpool."""

    resource = 'subnetpool'


class UpdateSubnetPool(neutronV20.UpdateCommand):
    """Update subnetpool's information."""

    resource = 'subnetpool'

    def add_known_arguments(self, parser):
        add_updatable_arguments(parser)
        parser.add_argument('--name',
                            help=_('Updated name of the subnetpool.'))
        addrscope_args = parser.add_mutually_exclusive_group()
        addrscope_args.add_argument('--address-scope',
                                    metavar='ADDRSCOPE',
                                    help=_('ID or name of the address scope '
                                           'with which the subnetpool is '
                                           'associated. Prefixes must be '
                                           'unique across address scopes.'))
        addrscope_args.add_argument('--no-address-scope',
                                    action='store_true',
                                    help=_('Detach subnetpool from the '
                                           'address scope.'))

    def args2body(self, parsed_args):
        body = {}
        updatable_args2body(parsed_args, body)

        # Parse and update for "address-scope" option/s
        if parsed_args.no_address_scope:
            body['address_scope_id'] = None
        elif parsed_args.address_scope:
            _addrscope_id = neutronV20.find_resourceid_by_name_or_id(
                self.get_client(), 'address_scope',
                parsed_args.address_scope)
            body['address_scope_id'] = _addrscope_id
        return {'subnetpool': body}



# Copyright 2012 VMware, Inc
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.common import exceptions
from neutronclient.neutron.v2_0 import router
from neutronclient.tests.unit import test_cli20


class CLITestV20RouterJSON(test_cli20.CLITestV20Base):
    def test_create_router(self):
        # Create router: router1.
        resource = 'router'
        cmd = router.CreateRouter(test_cli20.MyApp(sys.stdout), None)
        name = 'router1'
        myid = 'myid'
        args = [name, '--description', 'rooter']
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   description='rooter')

    def test_create_router_flavor(self):
        resource = 'router'
        cmd = router.CreateRouter(test_cli20.MyApp(sys.stdout), None)
        name = 'router1'
        myid = 'myid'
        flavor = 'router-flavor'
        args = [name, '--flavor', flavor]
        position_names = ['name', ]
        position_values = [name, flavor]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   flavor_id='router-flavor')

    def test_create_router_tenant(self):
        # Create router: --tenant_id tenantid myname.
        resource = 'router'
        cmd = router.CreateRouter(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        args = ['--tenant_id', 'tenantid', name]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_router_admin_state(self):
        # Create router: --admin_state_down myname.
        resource = 'router'
        cmd = router.CreateRouter(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        args = ['--admin_state_down', name, ]
        position_names = ['name', ]
        position_values = [name, ]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   admin_state_up=False)

    def _create_router_distributed_or_ha(self, distributed=None, ha=None):
        # Create router: --distributed distributed --ha ha myname.
        resource = 'router'
        cmd = router.CreateRouter(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        args = []
        if distributed is not None:
            args += ['--distributed', str(distributed)]
        if ha is not None:
            args += ['--ha', str(ha)]
        args.append(name)
        position_names = ['name', ]
        position_values = [name, ]
        expected = {}
        if distributed is not None:
            expected['distributed'] = str(distributed)
        if ha is not None:
            expected['ha'] = str(ha)
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   **expected)

    def test_create_router_distributed_True(self):
        # Create router: --distributed=True.
        self._create_router_distributed_or_ha(distributed='True')

    def test_create_router_ha_with_True(self):
        self._create_router_distributed_or_ha(ha='True')

    def test_create_router_ha_with_true(self):
        self._create_router_distributed_or_ha(ha='true')

    def test_create_router_ha_with_False(self):
        self._create_router_distributed_or_ha(ha='False')

    def test_create_router_ha_with_false(self):
        self._create_router_distributed_or_ha(ha='false')

    def test_create_router_distributed_False(self):
        # Create router: --distributed=False.
        self._create_router_distributed_or_ha(distributed='False')

    def test_create_router_distributed_true(self):
        # Create router: --distributed=true.
        self._create_router_distributed_or_ha(distributed='true')

    def test_create_router_distributed_false(self):
        # Create router: --distributed=false.
        self._create_router_distributed_or_ha(distributed='false')

    def test_create_router_with_az_hint(self):
        # Create router: --availability-zone-hint zone1
        # --availability-zone-hint zone2.
        resource = 'router'
        cmd = router.CreateRouter(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        args = ['--availability-zone-hint', 'zone1',
                '--availability-zone-hint', 'zone2', name]
        position_names = ['availability_zone_hints', 'name']
        position_values = [['zone1', 'zone2'], name]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_list_routers_detail(self):
        # list routers: -D.
        resources = "routers"
        cmd = router.ListRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_routers_pagination(self):
        resources = "routers"
        cmd = router.ListRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_routers_sort(self):
        # list routers:
        # --sort-key name --sort-key id --sort-key asc --sort-key desc
        resources = "routers"
        cmd = router.ListRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_routers_limit(self):
        # list routers: -P.
        resources = "routers"
        cmd = router.ListRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_update_router_exception(self):
        # Update router: myid.
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        self.assertRaises(exceptions.CommandError, self._test_update_resource,
                          resource, cmd, 'myid', ['myid'], {})

    def test_update_router(self):
        # Update router: myid --name myname --tags a b.
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--description', ':D'],
                                   {'name': 'myname', 'description': ':D'})

    def test_update_router_admin_state(self):
        # Update router: myid --admin-state-up <True|False>.
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--admin-state-up', 'True'],
                                   {'admin_state_up': 'True'}
                                   )
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--admin-state-up', 'true'],
                                   {'admin_state_up': 'true'}
                                   )
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--admin-state-up', 'False'],
                                   {'admin_state_up': 'False'}
                                   )
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--admin-state-up', 'false'],
                                   {'admin_state_up': 'false'}
                                   )

    def test_update_router_distributed(self):
        # Update router: myid --distributed <True|False>.
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--distributed', 'True'],
                                   {'distributed': 'True'}
                                   )
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--distributed', 'true'],
                                   {'distributed': 'true'}
                                   )
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--distributed', 'False'],
                                   {'distributed': 'False'}
                                   )
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--distributed', 'false'],
                                   {'distributed': 'false'}
                                   )

    def test_update_router_no_routes(self):
        # Update router: myid --no-routes
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--no-routes'],
                                   {'routes': None})

    def test_update_router_add_route(self):
        # Update router: myid --route destination=10.0.3.0/24,nexthop=10.0.0.10
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid,
                '--route',
                'destination=10.0.3.0/24,nexthop=10.0.0.10']
        routes = [{'destination': '10.0.3.0/24',
                   'nexthop': '10.0.0.10'}]
        updatefields = {'routes': routes}
        self._test_update_resource(resource, cmd, myid, args, updatefields)

    def test_update_router_add_routes(self):
        # Update router: myid --route destination=10.0.3.0/24,nexthop=10.0.0.10
        # --route destination=fd7a:1d63:2063::/64,
        #         nexthop=fd7a:1d63:2063:0:f816:3eff:fe0e:a697
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid,
                '--route',
                'destination=10.0.3.0/24,nexthop=10.0.0.10',
                '--route',
                'destination=fd7a:1d63:2063::/64,'
                'nexthop=fd7a:1d63:2063:0:f816:3eff:fe0e:a697']
        routes = [{'destination': '10.0.3.0/24',
                   'nexthop': '10.0.0.10'},
                  {'destination': 'fd7a:1d63:2063::/64',
                   'nexthop': 'fd7a:1d63:2063:0:f816:3eff:fe0e:a697'}]
        updatefields = {'routes': routes}
        self._test_update_resource(resource, cmd, myid, args, updatefields)

    def test_update_router_no_routes_with_add_route(self):
        # Update router: --no-routes with --route
        resource = 'router'
        cmd = router.UpdateRouter(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid,
                '--no-routes',
                '--route',
                'destination=10.0.3.0/24,nexthop=10.0.0.10']
        exception = self.assertRaises(SystemExit,
                                      self._test_update_resource,
                                      resource, cmd, myid, args, None)
        self.assertEqual(2, exception.code)

    def test_delete_router(self):
        # Delete router: myid.
        resource = 'router'
        cmd = router.DeleteRouter(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)

    def test_show_router(self):
        # Show router: myid.
        resource = 'router'
        cmd = router.ShowRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])

    def _test_add_remove_interface(self, action, mode, cmd, args):
        resource = 'router'
        subcmd = '%s_router_interface' % action
        if mode == 'port':
            body = {'port_id': 'portid'}
        else:
            body = {'subnet_id': 'subnetid'}
        if action == 'add':
            retval = {'subnet_id': 'subnetid', 'port_id': 'portid'}
            retval = self.client.serialize(retval)
            expected_code = 200
        else:
            retval = None
            expected_code = 204
        self._test_update_resource_action(resource, cmd, 'myid',
                                          subcmd, args,
                                          body, expected_code, retval)

    def test_add_interface_compat(self):
        # Add interface to router: myid subnetid.
        cmd = router.AddInterfaceRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'subnetid']
        self._test_add_remove_interface('add', 'subnet', cmd, args)

    def test_add_interface_by_subnet(self):
        # Add interface to router: myid subnet=subnetid.
        cmd = router.AddInterfaceRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'subnet=subnetid']
        self._test_add_remove_interface('add', 'subnet', cmd, args)

    def test_add_interface_by_port(self):
        # Add interface to router: myid port=portid.
        cmd = router.AddInterfaceRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'port=portid']
        self._test_add_remove_interface('add', 'port', cmd, args)

    def test_del_interface_compat(self):
        # Delete interface from router: myid subnetid.
        cmd = router.RemoveInterfaceRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'subnetid']
        self._test_add_remove_interface('remove', 'subnet', cmd, args)

    def test_del_interface_by_subnet(self):
        # Delete interface from router: myid subnet=subnetid.
        cmd = router.RemoveInterfaceRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'subnet=subnetid']
        self._test_add_remove_interface('remove', 'subnet', cmd, args)

    def test_del_interface_by_port(self):
        # Delete interface from router: myid port=portid.
        cmd = router.RemoveInterfaceRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'port=portid']
        self._test_add_remove_interface('remove', 'port', cmd, args)

    def test_set_gateway(self):
        # Set external gateway for router: myid externalid.
        resource = 'router'
        cmd = router.SetGatewayRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'externalid']
        self._test_update_resource(resource, cmd, 'myid',
                                   args,
                                   {"external_gateway_info":
                                    {"network_id": "externalid"}}
                                   )

    def test_set_gateway_enable_snat(self):
        # enable external gateway for router: myid externalid.
        resource = 'router'
        cmd = router.SetGatewayRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'externalid', '--enable-snat']
        self._test_update_resource(resource, cmd, 'myid',
                                   args,
                                   {"external_gateway_info":
                                    {"network_id": "externalid",
                                     "enable_snat": True}}
                                   )

    def test_set_gateway_disable_snat(self):
        # set external gateway for router: myid externalid.
        resource = 'router'
        cmd = router.SetGatewayRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'externalid', '--disable-snat']
        self._test_update_resource(resource, cmd, 'myid',
                                   args,
                                   {"external_gateway_info":
                                    {"network_id": "externalid",
                                     "enable_snat": False}}
                                   )

    def test_set_gateway_external_ip(self):
        # set external gateway for router: myid externalid --fixed-ip ...
        resource = 'router'
        cmd = router.SetGatewayRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'externalid', '--fixed-ip', 'ip_address=10.0.0.2']
        self._test_update_resource(resource, cmd, 'myid',
                                   args,
                                   {"external_gateway_info":
                                    {"network_id": "externalid",
                                     "external_fixed_ips": [
                                         {"ip_address": "10.0.0.2"}]}}
                                   )

    def test_set_gateway_external_subnet(self):
        # set external gateway for router: myid externalid --fixed-ip ...
        resource = 'router'
        cmd = router.SetGatewayRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'externalid', '--fixed-ip', 'subnet_id=mysubnet']
        self._test_update_resource(resource, cmd, 'myid',
                                   args,
                                   {"external_gateway_info":
                                    {"network_id": "externalid",
                                     "external_fixed_ips": [
                                         {"subnet_id": "mysubnet"}]}}
                                   )

    def test_set_gateway_external_ip_and_subnet(self):
        # set external gateway for router: myid externalid --fixed-ip ...
        resource = 'router'
        cmd = router.SetGatewayRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['myid', 'externalid', '--fixed-ip',
                'ip_address=10.0.0.2,subnet_id=mysubnet']
        self._test_update_resource(resource, cmd, 'myid',
                                   args,
                                   {"external_gateway_info":
                                    {"network_id": "externalid",
                                     "external_fixed_ips": [
                                         {"subnet_id": "mysubnet",
                                          "ip_address": "10.0.0.2"}]}}
                                   )

    def test_remove_gateway(self):
        # Remove external gateway from router: externalid.
        resource = 'router'
        cmd = router.RemoveGatewayRouter(test_cli20.MyApp(sys.stdout), None)
        args = ['externalid']
        self._test_update_resource(resource, cmd, 'externalid',
                                   args, {"external_gateway_info": {}}
                                   )



# Copyright (c) 2017 Huawei Technologies India Pvt.Limited.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib.cli import parseractions
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from osc_lib.utils import columns as column_util

from neutronclient._i18n import _

LOG = logging.getLogger(__name__)

resource = 'port_pair_group'

_attr_map = (
    ('id', 'ID', column_util.LIST_BOTH),
    ('name', 'Name', column_util.LIST_BOTH),
    ('port_pairs', 'Port Pair', column_util.LIST_BOTH),
    ('port_pair_group_parameters', 'Port Pair Group Parameters',
     column_util.LIST_BOTH),
    ('description', 'Description', column_util.LIST_LONG_ONLY),
    ('group_id', 'Loadbalance ID', column_util.LIST_LONG_ONLY),
    ('project_id', 'Project',  column_util.LIST_LONG_ONLY),
    ('tap_enabled', 'Tap Enabled', column_util.LIST_BOTH)
)


class CreateSfcPortPairGroup(command.ShowOne):
    _description = _("Create a port pair group")

    def get_parser(self, prog_name):
        parser = super(CreateSfcPortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the port pair group'))
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the port pair group'))
        parser.add_argument(
            '--port-pair',
            metavar='<port-pair>',
            dest='port_pairs',
            default=[],
            action='append',
            help=_('Port pair (name or ID). '
                   'This option can be repeated.'))
        tap_enable = parser.add_mutually_exclusive_group()
        tap_enable.add_argument(
            '--enable-tap',
            action='store_true',
            help=_('Port pairs of this port pair group are deployed as '
                   'passive tap service function')
        )
        tap_enable.add_argument(
            '--disable-tap',
            action='store_true',
            help=_('Port pairs of this port pair group are deployed as l3 '
                   'service function (default)')
        )
        parser.add_argument(
            '--port-pair-group-parameters',
            metavar='lb-fields=<lb-fields>',
            action=parseractions.KeyValueAction,
            help=_('Dictionary of port pair group parameters. '
                   'Currently only one parameter lb-fields is supported. '
                   '<lb-fields> is a & separated list of load-balancing '
                   'fields.'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        attrs = _get_common_attrs(self.app.client_manager, parsed_args)
        body = {resource: attrs}
        obj = client.create_sfc_port_pair_group(body)[resource]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


class DeleteSfcPortPairGroup(command.Command):
    _description = _("Delete a given port pair group")

    def get_parser(self, prog_name):
        parser = super(DeleteSfcPortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair_group',
            metavar='<port-pair-group>',
            help=_("Port pair group to delete (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        # TODO(mohan): Add support for deleting multiple resources.
        client = self.app.client_manager.neutronclient
        ppg_id = _get_id(client,  parsed_args.port_pair_group, resource)
        try:
            client.delete_sfc_port_pair_group(ppg_id)
        except Exception as e:
            msg = (_("Failed to delete port pair group with name "
                     "or ID '%(ppg)s': %(e)s")
                   % {'ppg': parsed_args.port_pair_group, 'e': e})
            raise exceptions.CommandError(msg)


class ListSfcPortPairGroup(command.Lister):
    _description = _("List port pair group")

    def get_parser(self, prog_name):
        parser = super(ListSfcPortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            '--long',
            action='store_true',
            default=False,
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        data = client.list_sfc_port_pair_groups()
        headers, columns = column_util.get_column_definitions(
            _attr_map, long_listing=parsed_args.long)
        return (headers,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data['port_pair_groups']))


class SetSfcPortPairGroup(command.Command):
    _description = _("Set port pair group properties")

    def get_parser(self, prog_name):
        parser = super(SetSfcPortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair_group',
            metavar='<port-pair-group>',
            help=_("Port pair group to modify (name or ID)"))
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the port pair group'))
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the port pair group'))
        parser.add_argument(
            '--port-pair',
            metavar='<port-pair>',
            dest='port_pairs',
            default=[],
            action='append',
            help=_('Port pair (name or ID). '
                   'This option can be repeated.'))
        parser.add_argument(
            '--no-port-pair',
            action='store_true',
            help=_('Remove all port pair from port pair group'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        ppg_id = _get_id(client, parsed_args.port_pair_group, resource)
        attrs = _get_common_attrs(self.app.client_manager, parsed_args,
                                  is_create=False)
        if parsed_args.no_port_pair:
            attrs['port_pairs'] = []
        if parsed_args.port_pairs:
            added = [client.find_resource('port_pair', pp,
                                          cmd_resource='sfc_port_pair')['id']
                     for pp in parsed_args.port_pairs]
            if parsed_args.no_port_pair:
                existing = []
            else:
                existing = client.find_resource(
                    resource, parsed_args.port_pair_group,
                    cmd_resource='sfc_port_pair_group')['port_pairs']
            attrs['port_pairs'] = sorted(list(set(existing) | set(added)))
        body = {resource: attrs}
        try:
            client.update_sfc_port_pair_group(ppg_id, body)
        except Exception as e:
            msg = (_("Failed to update port pair group '%(ppg)s': %(e)s")
                   % {'ppg': parsed_args.port_pair_group, 'e': e})
            raise exceptions.CommandError(msg)


class ShowSfcPortPairGroup(command.ShowOne):
    _description = _("Display port pair group details")

    def get_parser(self, prog_name):
        parser = super(ShowSfcPortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair_group',
            metavar='<port-pair-group>',
            help=_("Port pair group to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        ppg_id = _get_id(client, parsed_args.port_pair_group, resource)
        obj = client.show_sfc_port_pair_group(ppg_id)[resource]
        columns, display_columns = column_util.get_columns(obj, _attr_map)
        data = utils.get_dict_properties(obj, columns)
        return display_columns, data


class UnsetSfcPortPairGroup(command.Command):
    _description = _("Unset port pairs from port pair group")

    def get_parser(self, prog_name):
        parser = super(UnsetSfcPortPairGroup, self).get_parser(prog_name)
        parser.add_argument(
            'port_pair_group',
            metavar='<port-pair-group>',
            help=_("Port pair group to unset (name or ID)"))
        port_pair_group = parser.add_mutually_exclusive_group()
        port_pair_group.add_argument(
            '--port-pair',
            action='append',
            metavar='<port-pair>',
            dest='port_pairs',
            help=_('Remove port pair(s) from the port pair group '
                   '(name or ID). This option can be repeated.'))
        port_pair_group.add_argument(
            '--all-port-pair',
            action='store_true',
            help=_('Remove all port pairs from the port pair group'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.neutronclient
        ppg_id = _get_id(client, parsed_args.port_pair_group, resource)
        attrs = {}
        if parsed_args.port_pairs:
            existing = client.find_resource(
                resource, parsed_args.port_pair_group,
                cmd_resource='sfc_port_pair_group')['port_pairs']
            removed = [client.find_resource('port_pair', pp,
                       cmd_resource='sfc_port_pair')['id']
                       for pp in parsed_args.port_pairs]
            attrs['port_pairs'] = list(set(existing) - set(removed))
        if parsed_args.all_port_pair:
            attrs['port_pairs'] = []
        body = {resource: attrs}
        try:
            client.update_sfc_port_pair_group(ppg_id, body)
        except Exception as e:
            msg = (_("Failed to unset port pair group '%(ppg)s': %(e)s")
                   % {'ppg': parsed_args.port_pair_group, 'e': e})
            raise exceptions.CommandError(msg)


def _get_ppg_param(attrs, ppg):
    attrs['port_pair_group_parameters'] = {}
    for key, value in ppg.items():
        if key == 'lb-fields':
            attrs['port_pair_group_parameters']['lb_fields'] = ([
                field for field in value.split('&') if field])
        else:
            attrs['port_pair_group_parameters'][key] = value
    return attrs['port_pair_group_parameters']


def _get_common_attrs(client_manager, parsed_args, is_create=True):
    attrs = {}
    if parsed_args.name is not None:
        attrs['name'] = parsed_args.name
    if parsed_args.description is not None:
        attrs['description'] = parsed_args.description
    if parsed_args.port_pairs:
        attrs['port_pairs'] = [(_get_id(client_manager.neutronclient, pp,
                                        'port_pair'))
                               for pp in parsed_args.port_pairs]
    if is_create:
        _get_attrs(attrs, parsed_args)
    return attrs


def _get_attrs(attrs, parsed_args):
    if parsed_args.port_pair_group_parameters is not None:
        attrs['port_pair_group_parameters'] = (
            _get_ppg_param(attrs, parsed_args.port_pair_group_parameters))
    if parsed_args.enable_tap:
        attrs['tap_enabled'] = True
    if parsed_args.disable_tap:
        attrs['tap_enabled'] = False


def _get_id(client, id_or_name, resource):
    return client.find_resource(resource, id_or_name)['id']
